from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class OccurenceInfo(TcBaseObj):
    """
    The structure contains information that represents occurrence.
    
    :var occThreadsChain: The uid strings of occurrence threads tha represent occurrences in the tree.
    :var stockAbleOccThread: The stock able occurrence - highest occurence level for setting IDIC. This parameter is
    optional and is used for perfomance reason - minimize the number of object to set IDIC. If stock able occurrence is
    received, IDIC is created for this occurence and remaining occurrence thread chain is concatenated with IDIC and
    returned to the client.
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements.
    """
    occThreadsChain: List[str] = ()
    stockAbleOccThread: str = ''
    clientID: str = ''


@dataclass
class OccurrenceIDIC(TcBaseObj):
    """
    The structure contains IDIC uid string of occurrence.
    
    :var returnID: The new created IDIC. If stockAbleOccThread is received, the server returns a new IDIC for the stock
    able occurrence , concatenated with the remaining occurrence thread IDs in the following way:
    returnID = IDIC_of_stockAbleOcc/cloneStableID_of_occthread1/cloneStableID_of_occthread2/cloneStableID_of_occthread3
    :var clientID: A unique string supplied by the caller. This ID is used to identify return data elements.
    """
    returnID: str = ''
    clientID: str = ''


@dataclass
class PostAssignIDICInput(TcBaseObj):
    """
    The structure contains input parameters: PMI context and occurrences from this context.
    
    :var context: The BOMView of the context that contains received occurrences.
    :var occurrences: The list with occurrences.
    """
    context: BusinessObject = None
    occurrences: List[OccurenceInfo] = ()


@dataclass
class PostAssignIDICResponse(TcBaseObj):
    """
    The stucture contains the return parameters: the new created IDICs and  service data with errors
    
    :var occurrenceIDICs: The list of structures with created IDIC.
    :var serviceData: Holds partial errors.
    """
    occurrenceIDICs: List[OccurrenceIDIC] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ChangeItemRevision, WorkspaceObject, ImanRelation
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateChangeLineageInputData(TcBaseObj):
    """
    Input data for creation of lineage.
    
    :var clientId: This unique ID is used to identify return data elements and partial errors associated with this
    input structure.
    :var solutionItems: List of WorkspaceObject which are referred as Solution Items for the ChangeNoticeRevision. All
    the solution items must belong to same ChangeNoticeRevision.
    :var impactedItems: List of WorkspaceObject which are referred as Impacted Items for the ChangeNoticeRevision. All
    the impacted items must belong to same ChangeNoticeRevision.
    """
    clientId: str = ''
    solutionItems: List[WorkspaceObject] = ()
    impactedItems: List[WorkspaceObject] = ()


@dataclass
class CreateChangeLineageOutput(TcBaseObj):
    """
    Change Lineage output.
    
    :var clientId: The unmodified value from the CreateChangeLineageInputData.clientId. This can be used by the caller
    to identify this data structure with the source input data.
    :var relations: A list of created relations.
    """
    clientId: str = ''
    relations: List[ImanRelation] = ()


@dataclass
class CreateChangeLineageResponse(TcBaseObj):
    """
    Change Lineage creation response.
    
    :var serviceData: Service data.
    :var output: Change Lineage output.
    """
    serviceData: ServiceData = None
    output: List[CreateChangeLineageOutput] = ()


@dataclass
class DeleteChangeLineageInputData(TcBaseObj):
    """
    DeleteChangeLineageInputData structure contains clientId to uniquely identify the input, an object reference that
    can be used to point to the ChangeNoticeRevision and list to hold any number of business objects for which change
    lineage has to be deleted.
    
    :var clientId: Input string to uniquely identify the input.
    :var changeNoticeRev: The ChangeNoticeRevision associated with the lineage to be deleted.
    :var objects: A list of objects for which the related lineage will be deleted.
    """
    clientId: str = ''
    changeNoticeRev: ChangeItemRevision = None
    objects: List[WorkspaceObject] = ()



from __future__ import annotations

from tcsoa.gen.Query._2010_09.SavedQuery import BusinessObjectQueryInput
from typing import List
from tcsoa.gen.Query._2007_09.SavedQuery import SavedQueriesResponse
from tcsoa.base import TcService


class SavedQueryService(TcService):

    @classmethod
    def executeBusinessObjectQueries(cls, inputs: List[BusinessObjectQueryInput]) -> SavedQueriesResponse:
        """
        Execute business object searches (Simple Search) and return search results.
        
        Use cases:
        Execute business object searches (Simple Search).
        """
        return cls.execute_soa_method(
            method_name='executeBusinessObjectQueries',
            library='Query',
            service_date='2010_09',
            service_name='SavedQuery',
            params={'inputs': inputs},
            response_cls=SavedQueriesResponse,
        )



from __future__ import annotations

from tcsoa.gen.Internal.Core._2017_11.LogicalObject import LogicalObjectTypeResponse, LogicalObjectTypeInput, AddMembersPresentedPropsInput, DeleteMembersPresentedPropsInput, AddMemberAndPresentedPropsResponse
from tcsoa.gen.BusinessObjects import BusinessObject
from tcsoa.gen.Internal.Core._2017_11.Type import GetSubTypeHierarchicalTreeInput, GetSubTypeHierarchicalTreeResponse
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.gen.Internal.Core._2017_11.DataManagement import TCSessionAndAnalyticsInfo
from tcsoa.base import TcService


class TypeService(TcService):

    @classmethod
    def getSubTypeHierarchicalTrees(cls, givenTypes: List[GetSubTypeHierarchicalTreeInput]) -> GetSubTypeHierarchicalTreeResponse:
        """
        This operation returns hierarchical tree structures of sub types for given types.
        """
        return cls.execute_soa_method(
            method_name='getSubTypeHierarchicalTrees',
            library='Internal-Core',
            service_date='2017_11',
            service_name='Type',
            params={'givenTypes': givenTypes},
            response_cls=GetSubTypeHierarchicalTreeResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def getTCSessionAnalyticsInfo(cls, extraInfoIn: List[str]) -> TCSessionAndAnalyticsInfo:
        """
        This operation provides information about the current user&rsquo;s Teamcenter session and data required for
        Teamcenter Software Analytics processing.
        
        Use cases:
        After the user logs into Active Workspace this operation is invoked by the Teamcenter client to get information
        related to the user session like Groups, Projects, the Teamcenter Server version, platform type,License type,
        preference related to the time interval and product excellence agreement as required by analytics.
        """
        return cls.execute_soa_method(
            method_name='getTCSessionAnalyticsInfo',
            library='Internal-Core',
            service_date='2017_11',
            service_name='DataManagement',
            params={'extraInfoIn': extraInfoIn},
            response_cls=TCSessionAndAnalyticsInfo,
        )


class LogicalObjectService(TcService):

    @classmethod
    def addMembersAndPresentedProps(cls, addMembersAndPresentedProps: AddMembersPresentedPropsInput) -> AddMemberAndPresentedPropsResponse:
        """
        This operation adds a list of members and presented properties to an existing Logical Object type based on the
        specified input.
        
        Use cases:
        This operation is invoked to add members and presented properties to an already defined logical object type
        based on input values.
        """
        return cls.execute_soa_method(
            method_name='addMembersAndPresentedProps',
            library='Internal-Core',
            service_date='2017_11',
            service_name='LogicalObject',
            params={'addMembersAndPresentedProps': addMembersAndPresentedProps},
            response_cls=AddMemberAndPresentedPropsResponse,
        )

    @classmethod
    def createLogicalObjectTypes(cls, logicalObjectTypeInputs: List[LogicalObjectTypeInput]) -> LogicalObjectTypeResponse:
        """
        This operation creates a list of logical object types based on the specified input. The name, display name,
        description and root type are the mandatory fields required to create a logical object type. A logical object
        type is created only if its definition data satisfies the pre-defined constraints.
        
        Use cases:
        This operation is invoked to create logical object types after obtaining the user input fields from the logical
        object creation dialog.
        
        Use Case 1:
        The user wants to create a logical object type with a root business object and without member and presented
        properties.
        
        In this case the input structure to create a logical object type would look like the following
        
        LogicalObjectTypeInput = {
        name                = loName;
        displayName   = logicalObjectName;
        description      = Logical Object Type;
        rootType          = Item;
        parentType      = Fnd0LogicalObject;
        }
        
        Use Case 2:
        The user wants to create a logical object type with root business object, member and presented properties as
        below
        
        1.  root business object    :   Item
        2.  member object            :   ItemRevision
        3.  presented properties   :   item_revision_id and item_id.
        
        
        In this case the input structures to create a logical object type with member and presented property
        definitions would look like the following
        
        LogicalObjectTypeInput = {
        name                                             = loName;
        displayName                                = logicalObjectName;
        description                                   = Logical Object Type;
        rootType                                       = Item;
        parentType                                   = Fnd0LogicalObject;
        memberPropertyDefinitions      = exampleMemberPropertyDefinitions;
        presentedPropertyDefinitions    = examplePresentedPropertyDefinitions;
        }
        
        exampleMemberPropertyDefinitions[0] = {
        memberPropertyName     = itemRevisionMember;
        displayName                       = Item Revision;
        description                          = Item Revision members;
        traversalPath                       = exampleTraversalPaths;
        }
        
        exampleTraversalPaths [0] = {
        propertyName       = fnd0Root
        propertyType         = refrence
        destinationType     = Item
        direction                  = forward
        }
        exampleTraversalPaths [1] = {
        propertyName        = items_tag
        propertyType          = reference
        destinationType     = ItemRevision
        direction                  = reverse
        }
        
        examplePresentedPropertyDefinitions[0] = {
        presentedPropertyName = ItemRevisionId;
        displayName                     = ItemRevisionId;
        description                        = Presented property from member: ItemRevissionId;
        rootOrMemberName      = itemRevisionMember;
        sourcePropertyName      = item_revision_id;
        }
        examplePresentedPropertyDefinitions[1] = {
        presentedPropertyName   = ItemId;
        displayName                       = ItemId;
        description                          = Presented property from root: ItemId;
        rootOrMemberName        = fnd0Root;
        sourcePropertyName        = item_id;
        }
        """
        return cls.execute_soa_method(
            method_name='createLogicalObjectTypes',
            library='Internal-Core',
            service_date='2017_11',
            service_name='LogicalObject',
            params={'logicalObjectTypeInputs': logicalObjectTypeInputs},
            response_cls=LogicalObjectTypeResponse,
        )

    @classmethod
    def deleteLogicalObjectTypes(cls, logicalObjects: List[BusinessObject]) -> ServiceData:
        """
        This operation deletes a list of logical object definitions and its member and presented property definitions.
        """
        return cls.execute_soa_method(
            method_name='deleteLogicalObjectTypes',
            library='Internal-Core',
            service_date='2017_11',
            service_name='LogicalObject',
            params={'logicalObjects': logicalObjects},
            response_cls=ServiceData,
        )

    @classmethod
    def deleteMembersAndPresentedProps(cls, deleteMembersAndPresentedProps: List[DeleteMembersPresentedPropsInput]) -> ServiceData:
        """
        This operation deletes a list of logical object member definitions and/or a list of logical object presented
        property definitions or a list of included logical object definitions. When a logical object member is deleted
        all presented properties that are from this member are also implicitly deleted.
        """
        return cls.execute_soa_method(
            method_name='deleteMembersAndPresentedProps',
            library='Internal-Core',
            service_date='2017_11',
            service_name='LogicalObject',
            params={'deleteMembersAndPresentedProps': deleteMembersAndPresentedProps},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import InstalledTemplates
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class SiteTemplateDeployInfoResponse(TcBaseObj):
    """
    The response data for the 'getSiteTemplateDeployInfo' operation.
    
    :var siteID: Site ID of the current database
    :var siteName: Site name of the current database
    :var serviceData: Status of the operation.
    :var templatesDeployInfo: list of 'InstalledTemplate' objects
    """
    siteID: str = ''
    siteName: str = ''
    serviceData: ServiceData = None
    templatesDeployInfo: List[InstalledTemplates] = ()


@dataclass
class DeployDataModelResponse(TcBaseObj):
    """
    This is the response that the service returns to the caller of the 'deployDataModel2'. It contains the info about
    the status of the operation, logfiles, the site and the related data.
    
    :var siteID: The site ID of the database to which the connection is established.
    
    :var siteName: The name of the database site to which connection is established.
    :var templateDeployInfo: the 'TemplateDeployInfo' object containing details from 'InstalledTemplates' table for the
    deployed template in the Database.
    :var serviceData: A 'ServiceData' structure containing the status of the operation execution
    :var logFileTickets: List of file tickets for the output log file(s) showing the results of the business model
    updater
    """
    siteID: str = ''
    siteName: str = ''
    templateDeployInfo: InstalledTemplates = None
    serviceData: ServiceData = None
    logFileTickets: List[str] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class SessionService(TcService):

    @classmethod
    def disableUserSessionState(cls, names: List[str]) -> ServiceData:
        """
        Remove the named User Session State variables from the client context, and make the Teamcenter Server that
        master for these state variables. All client applications sharing the instance of the Teamcenter Server will
        use the current values of the named state variables.   The current value of the state variable as defined by
        this calling client application will become the master value for all clients.
        """
        return cls.execute_soa_method(
            method_name='disableUserSessionState',
            library='Internal-Core',
            service_date='2008_03',
            service_name='Session',
            params={'names': names},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, BusinessObject, VariantRule, Variant, BOMWindow, Item
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ApplyBOMVariantRulesResponse(TcBaseObj):
    """
    It contains list of variant rules applied on the window.
    
    :var rules: List of  BOMVariantRuleContents object associated with the BOMWidow
    :var serviceData: Object of service data, that returns partial errors.
    :var window: BOMWindow object on which rules have been applied
    """
    rules: List[BOMVariantRuleContents] = ()
    serviceData: ServiceData = None
    window: BOMWindow = None


@dataclass
class GetBOMVariantRuleInput(TcBaseObj):
    """
    It contains information about window and saved variant rules which user want to add, delete, update or override.
    
    :var window: BOMWindow object for which variant rule is being requested.
    :var variantRule: The variant rule for which the contents are requested.
    :var svrs: List of saved variant rules which user want to add, unset, update or override. It is optional.
    :var clientId: unique identifier for each window. Output will have same identifier, to let caller know, which rule
    list is associated with which window.
    :var svrActionMode: This flag indicates different action mode associated to SVRs which includes  add (add new saved
    variant rule), unset (unset existing saved variant rule), update (update configuration using saved variant rule),
    override (override configuration using saved variant rule). 
    
    0     Set this value, when you want to get variant rule from window(default value).
    1      Set this value, when you want to apply or add new saved variant rule in case of multiple rule scenario
    2      Set this value, when you want to unset existing saved variant rule in case of multiple rule scenario. 
    3     Set this mode, if user wants to override existing option value in saved variant rule.
    4     Set this mode, if user wants to update existing option value in saved variant rule.
    """
    window: BOMWindow = None
    variantRule: VariantRule = None
    svrs: List[BusinessObject] = ()
    clientId: str = ''
    svrActionMode: int = 0


@dataclass
class GetVariantExpressionsMatchInfoResponse(TcBaseObj):
    """
    The response of method getVariantExpressionsMatchInfo.
    
    :var variantExpressionsDetails: The entries in this list corresponds to each input InputBOMLinesSet.
    :var serviceData: The partial errors.
    """
    variantExpressionsDetails: List[VariantExpressionsDetails] = ()
    serviceData: ServiceData = None


@dataclass
class InputBOMLinesSet(TcBaseObj):
    """
    The list of roll up BOMLine objects and non-roll up BOMLine objects.
    
    :var rollUpBOMLines: The list of BOMLine objects for which rolled up variant conditions and rolled up clause lists 
    to be calculated. If this list is empty, then this operation will populate the variant expressions for the
    nonRollUPBOMLines and their corresponding clause lists in the response.
    :var nonRollUpBOMLines: The list of BOMLine objects for which occurrence variant conditions and clause lists to be
    returned. If this list is empty, , then this operation will populate the rolled up variant expressions and
    corresponding clause lists in the response.
    :var doCompare: If true, the comparison will be peformed between all the input bomLines i.e. rollUpBOMLines and
    nonRollUpBOMLines till a mismatch is found.
    """
    rollUpBOMLines: List[BOMLine] = ()
    nonRollUpBOMLines: List[BOMLine] = ()
    doCompare: bool = False


@dataclass
class BOMVariantOptionValueEntry(TcBaseObj):
    """
    It contains details of option, its description and values associated with that particular option.
    
    :var option: Variant object that contains details of option value.
    :var owningItem: Item on which option is set.
    :var optionName: Name of the option associated with Item
    :var optionDesc: Additional information about option
    :var variantOptionValue: List containing options and configuration details
    """
    option: Variant = None
    owningItem: Item = None
    optionName: str = ''
    optionDesc: str = ''
    variantOptionValue: List[VariantOptionValue] = ()


@dataclass
class BOMVariantRuleContents(TcBaseObj):
    """
    BOMVariantRuleContents contains variant rule, the saved variant rules associated with the window,   list of option
    value details and a boolean flag for constraints evaluation status.
      
    In getBOMVariantRules operation , this structure used as output. 
    
    In setBOMVariantRule operation, this structure used as input as well as output. As input, it contains details of
    variant rule, saved variant rule, saved variant rule modification flag and list of options that needs to be set on
    variant rule. As output, it contains details of variant rule with set option values. 
    
    In applyBOMVariantRules operation, this structure used as input as well as output. Typically the variantRule, svr,
    isSVRModified provided as input and  list of BOMVariantOptionValueEntry and isConstraintsEvaluated are part of
    output.  
     
    Constraints (defaults, derived defaults and rule check) will only be evaluated for single variant rule having
    single option value.
    
    
    :var variantRule: VariantRule with which window has been configured. This cannot be NULL
    :var svr: Saved variant rule with which  window has been configured. This will be NULL if window is not configured
    with saved variant rule.
    :var isSVRModified: True, if saved variant rule has been modified.
    :var bomVariantOptionValueEntry: List of options associated with VariantRule 
    :var isConstraintsEvaluated: True, if default, derived default have been evaluated
    """
    variantRule: VariantRule = None
    svr: BusinessObject = None
    isSVRModified: bool = False
    bomVariantOptionValueEntry: List[BOMVariantOptionValueEntry] = ()
    isConstraintsEvaluated: bool = False


@dataclass
class BOMVariantRuleOutput(TcBaseObj):
    """
    BOMVariantRuleOutput contains window and the list of rule associated with window. It has the identifier to map
    input variant rule to output variant rule.
    
    :var window: BOMWindow on which rules are set
    :var rules: List of  BOMVariantRuleContents  objects
    :var clientId: Identifier to map input variant rules to output variant rules.
    """
    window: BOMWindow = None
    rules: List[BOMVariantRuleContents] = ()
    clientId: str = ''


@dataclass
class BOMVariantRulesResponse(TcBaseObj):
    """
    List of varaint rule data associated with window.
    
    :var variantRuleData: List of  BOMVariantRuleOutput  object
    :var serviceData: Object of service data.
    """
    variantRuleData: List[BOMVariantRuleOutput] = ()
    serviceData: ServiceData = None


@dataclass
class SetBOMVariantRuleData(TcBaseObj):
    """
    SetBOMVariantRuleData contains information about window and variant rule that needs to be set on the window. This
    structure will be used as an input and as an output. Input contains details of  window and BOMVariantRuleContents 
    that needs to be set on window . Output contains details of BOMVariantRuleContents  having information about option
    values set on the variant rule.
    
    :var window: BOMWindow object on which variant rules will be set
    :var rules: A list of BOMVariantRuleContents  object which contain details of Variant Rule and list of options and
    values.
    :var clientId: Identifier to map input variant rule to output variant rule
    """
    window: BOMWindow = None
    rules: List[BOMVariantRuleContents] = ()
    clientId: str = ''


@dataclass
class SetBOMVariantRulesResponse(TcBaseObj):
    """
    SetBOMVariantRulesResponse object reference.
    
    :var setBOMVariantRuleData: List of  objects containing setBOMVariantRuleData  object 
    :var serviceData: Object of service data that that returns partial errors
    """
    setBOMVariantRuleData: List[SetBOMVariantRuleData] = ()
    serviceData: ServiceData = None


@dataclass
class VariantExpressionClauseList(TcBaseObj):
    """
    Contains clauseList object and corresponding clause list text.
    
    :var clauseListText: The text representation of the clauseList. If the line does not have the variant condition
    then this list will be empty.
    :var clauseList: The clauseList object of the VarientExpression. If the line does not have the variant condition
    then corresponding entry in this list will be null.
    """
    clauseListText: List[str] = ()
    clauseList: BusinessObject = None


@dataclass
class VariantExpressionsDetails(TcBaseObj):
    """
    The VariantExpressionsDetails represents response for each input InputBOMLinesSet. It contains clause List and
    corresponding clause List text for rollUpBOMLines and nonRollUpBOMLines.
    
    :var rollUpClauseList: The rolled up clause list object and clauseListText for the BOMLine objects in the
    rollUpBomLines list. Each entry in this list corresponds to one BOMLine in rollUpBomLines list.
    :var nonRollUpClauseList: The clause lists for the BOMLines objects in the nonRollUpBomLines list.
    :var isDifferent: True if the input doCompare flag is set and any of the BOMLines of the inputBOMLinesSets are
    different due to variant conditions.
    """
    rollUpClauseList: List[VariantExpressionClauseList] = ()
    nonRollUpClauseList: List[VariantExpressionClauseList] = ()
    isDifferent: bool = False


@dataclass
class VariantOptionValue(TcBaseObj):
    """
    Details of option values.
    
    :var value: Value assigned to option 
    :var howSet: Indicates, how value is set. Possible values are  
    -     Unset  value is unset, 
    -     Unset (potentially derived)  derivable value is unset,
    -     Derived  value is from derived option value  ,
    -     Defaulted  value from default option value, 
    -     Set by User  value is set by user, 
    -     Set by Rule  value is set by rule,
    -     Variant Item  value set by variant item
    
    
    :var howSetInt: It indicates how value is set. It is integer representation of  howSet. 
    Valid values are as follows. 
    0  value is unset, 
    1  derivable value is unset,
    2  value is from derived option value  ,
    3  value from default option value, 
    4  value is set by user, 
    5  value is set by rule,
    8  value set by variant item.
    
    :var whereSet: Name of saved variant rule given by user or name of the owningItem
    :var index: Position of option value in variant revision
    :var isConfiguredOptionValue: If rule is configured with particular option value.
    """
    value: str = ''
    howSet: str = ''
    howSetInt: int = 0
    whereSet: str = ''
    index: int = 0
    isConfiguredOptionValue: bool = False



from __future__ import annotations

from tcsoa.gen.RelationshipViewer._2019_12.NetworkEngine import GetViewsParamMap
from tcsoa.gen.RelationshipViewer._2014_11.NetworkEngine import GraphTypeListResponse
from tcsoa.base import TcService


class NetworkEngineService(TcService):

    @classmethod
    def getViews4(cls, getViewsInput: GetViewsParamMap) -> GraphTypeListResponse:
        """
        This operation provides a list of the available graph view types identified by the input key and the object
        type. A graph view is a set of configuration that can be used for network expansion. For example: localized
        view name and category types which classify a list of supported Teamcenter Business Object types, legend color
        of each category types. The view list is role based, different role may get different view list.
        """
        return cls.execute_soa_method(
            method_name='getViews4',
            library='RelationshipViewer',
            service_date='2019_12',
            service_name='NetworkEngine',
            params={'getViewsInput': getViewsInput},
            response_cls=GraphTypeListResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject, PSBOMView, WorkspaceObject
from tcsoa.gen.Core._2007_06.DataManagement import DatasetTypeInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetDatasetTypesWithFileExtensionOutput(TcBaseObj):
    """
    This struct contains pair of file extension and the list of DatasetTypeInfo structures.
    
    :var fileExtension: The fileExtension for each dataset type specified in fileExtensions input
    :var datasetTypesInfo: The matching list of named reference information
    """
    fileExtension: str = ''
    datasetTypesInfo: List[DatasetTypeInfo] = ()


@dataclass
class GetDatasetTypesWithFileExtensionResponse(TcBaseObj):
    """
    This struct contains the list dataset type and reference information and Service Data.
    
    :var output: List of named reference information for each dataset type specified in fileExtensions input
    :var serviceData: The DatasetType objects that corresponds to fileExtensions input
    """
    output: List[GetDatasetTypesWithFileExtensionOutput] = ()
    serviceData: ServiceData = None


@dataclass
class TraceReport1(TcBaseObj):
    """
    This structure holds information about generated trace reports with defining and complying trace trees with
    selected objects, and its persistent objects.
    
    :var defRootNode: Represents the root node of Defining Tree in the Trace Report.
    :var compRootNode: Represents the root node of Complying Tree in the Trace Report.
    :var selectedObject: Represents selected object for which Trace Report generated.
    :var persistentObjects: Represents persistent object for the selected object for which Trace Report generated.
    """
    defRootNode: TraceReportTreeNode = None
    compRootNode: TraceReportTreeNode = None
    selectedObject: BusinessObject = None
    persistentObjects: List[BusinessObject] = ()


@dataclass
class TraceReportTreeNode(TcBaseObj):
    """
    This structure holds information of each trace link report line with information of parent object, and its children
    associated with trace link relation
    
    :var object: This object represents any BusinessObject in Teamcenter on which Trace Link is created.
    :var displayObj: If the Trace Link is created on the occurrence then this variable will represent the object to
    show in the Trace Report.
    :var srcContextName: This parameter will hold the name of context for source object of Trace Link.
    :var tarContextName: This parameter will hold the name of context for target object of Trace Link.
    :var bomView: This field represents the BOMView of context line for the TraceLink.
    :var isDirectLink: This flag tells if current object is direct trace link object or not.
    :var isTraceLinkObj: This flag tells if current object is trace link object or not.
    :var childNodes: This represents child nodes with trace link relation with current object.
    """
    object: BusinessObject = None
    displayObj: WorkspaceObject = None
    srcContextName: str = ''
    tarContextName: str = ''
    bomView: PSBOMView = None
    isDirectLink: bool = False
    isTraceLinkObj: bool = False
    childNodes: List[TraceReportTreeNode] = ()


@dataclass
class TraceabilityFilterInput(TcBaseObj):
    """
    TraceabilityFilterInput structure has parameters for property filtering for trace report building. This includes
    name of property on which filter will get applied, operator type, and the value of comparison.
    
    :var logicalOperatorType: Type of the operator applied to concatenate this property clause with previous property
    clause. Below are valid values
    0 - LOGICAL_AND  
    1 - LOGICAL_OR
    :var propertyName: Name of the property to be filtered.
    :var operatorType: Type of operator to be applied to compare the value of property, below are valid values 
                            0 = OP_EQUALS
                            1 = OP_NOT_EQUALS
                             2 = OP_LESS_THAN
                            3 = OP_LESS_OR_EQUAL
                            4 = OP_GREATER_THAN
                            5 = OP_GREATER_OR_EQUAL
    
    :var propertyValue: Value of property expected.
    """
    logicalOperatorType: int = 0
    propertyName: str = ''
    operatorType: int = 0
    propertyValue: str = ''


@dataclass
class TraceabilityInfoInput1(TcBaseObj):
    """
    Information required to generate a trace report.  Type of trace report, if indirect trace link included, and list
    of FND_TraceLink relation type name and object type names to be included in trace report.  Also, it includes all
    the details of apply property filtering and name of property by which trace report needs to updated. Also it has
    details about whether to export trace report to RAC to show or to export to MSExcel application.
    
    :var selectedObjs: List of objects on which to generate the Trace Report.
    :var reportType: Type of Report that is defining(0), complying(1), or both(2).
    :var exportTo: Format name to export trace report. It will be either "TraceReportRACExport" - for exporting
    Traceability report to RAC, or " TraceReportMSExcelExport" - for exporting Traceability report to MSExcel
    application. 
    
    :var exportTemplate: Name of export template to be used to export the trace report.
    :var exportColumnNames: This is list of real property names to be exported to Excel.
    :var reportDepth: Level to which Trace Report should be generated. Its value should be greater than 0.
    :var isIndirectTraceReportNeeded: If true then only Indirect Trace Report will be included in the final Trace
    Report.
    :var filteredTraceLinkTypes: This is the list of FND_TraceLink type, for which type only the Trace Report needs to
    be constructed excluding other types. 
    :var filteredObjectTypes: The list of object type, for which type only the Trace Report needs to be constructed
    excluding other types.
    :var scopeLines: The list of all the scope BOM lines from BOM Window under which the searched object (which is
    going to be added in trace report ) BOM lines needs to be searched.
    :var propertyFilterInput: The list of property filtering inputs. This property filter needs to be applied of the
    trace report going to send to client and only filter criteria report will be sent to the client.
    :var sortPropName: Name of the property name by which the Trace Report needs to be sorted.
    :var sortDirection: 1 to sort ascending order, 2 to sort descending order. 0 if no sort applied.
    """
    selectedObjs: List[BusinessObject] = ()
    reportType: int = 0
    exportTo: str = ''
    exportTemplate: str = ''
    exportColumnNames: List[str] = ()
    reportDepth: int = 0
    isIndirectTraceReportNeeded: bool = False
    filteredTraceLinkTypes: List[str] = ()
    filteredObjectTypes: List[str] = ()
    scopeLines: List[BusinessObject] = ()
    propertyFilterInput: List[TraceabilityFilterInput] = ()
    sortPropName: str = ''
    sortDirection: int = 0


@dataclass
class TraceabilityReportOutput1(TcBaseObj):
    """
    This structure holds information of either of following, including service data:
    1. Generated trace reports with defining and/or complying trace trees with selected objects, and its persistent
    objects
    2. Information about the file ticket for exported file generated at the server.
    
    :var traceReports: This member holds all the Trace Reports user has asked for. This is vector of TraceReport1 type
    of structures.
    :var transientFileReadTickets: The transient file read tickets for the exported file.
    :var serviceData: Service Data.
    """
    traceReports: List[TraceReport1] = ()
    transientFileReadTickets: List[str] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetObjectsLockInfoResponse(TcBaseObj):
    """
    GetObjectsLockInfoResponse structure defines the response from getObjectsLockInfo operation. It contains the
    information of the found locks. It provides the errors if the operation fails too.
    
    :var lockInfo: The list of found lock information. It includes:
    1.    Business object UID.
    2.    Lock date and time.
    3.    Lock user id.
    4.    Lock node name.
    :var serviceData: Service data contains the errors if the operation fails.
    """
    lockInfo: List[MarkOTLockInfo] = ()
    serviceData: ServiceData = None


@dataclass
class MarkOTErrorInfo(TcBaseObj):
    """
    MarkOTErrorInfo structure represents the failure information for adding marks.
    
    :var objectUID: Business object UID.
    :var errorInfo: The error message for the business object. 
    It includes:
    1.    Error code from the server side.
    2.    Error message for the error code.
    """
    objectUID: str = ''
    errorInfo: List[NameAndValue] = ()


@dataclass
class MarkOTInfo(TcBaseObj):
    """
    MarkOTInfo structure represents the mark information for briefcase ownership transfer.
    
    :var objectUID: Business object UID.
    :var targetSiteId: Target site id for ownership transfer.
    :var userId: User id who locks the object.
    :var parentUID: The anchor business object. It is used by ItemRevision objects only and the value is its Item. The
    value is empty for other objects.
    """
    objectUID: str = ''
    targetSiteId: int = 0
    userId: str = ''
    parentUID: str = ''


@dataclass
class MarkOTLockInfo(TcBaseObj):
    """
    MarkOTLockInfo structure represents the lock information.
    
    :var objectUID: Business object UID.
    :var lockUserId: User id who locks the object.
    :var lockTime: Lock time.
    :var lockNodeName: Lock node name.
    """
    objectUID: str = ''
    lockUserId: str = ''
    lockTime: str = ''
    lockNodeName: str = ''


@dataclass
class NameAndValue(TcBaseObj):
    """
    NameAndValue structure represents a generic name-value pair.
    
    :var elementName: The name of the name-values pair.
    :var elementValue: The value of the name-values pair.
    """
    elementName: str = ''
    elementValue: str = ''


@dataclass
class QueryMarkOTResponse(TcBaseObj):
    """
    QueryMarkOTResponse structure defines the response from queryMarkOT operation. It contains the details of each
    marks. It provides errors if the operation fails too.
    
    :var markInfo: A list of found marks. 
    The information includes:
    1.    Business object UID.
    2.    Target site id.
    3.    Lock user id.
    4.    The anchor business object. This is used by ItemRevision and the value is its Item.
    :var serviceData: Service data contains the errors if the operation fails.
    """
    markInfo: List[MarkOTInfo] = ()
    serviceData: ServiceData = None


@dataclass
class RemoveMarkOTResponse(TcBaseObj):
    """
    RemoveMarkOTResponse structure defines the response from removeMarkOTForCurrentUser operation. It contains the
    success and failure information of each objects for removing marks. It provides the errors if the operation fails
    too.
    
    :var successInfo: The list of the success business object UID.
    :var failureInfo: The list of failure information. It includes:
    1.    Business object.
    2.    Error code.
    3.    Error message.
    :var serviceData: Service data contains the errors if the operation fails.
    """
    successInfo: List[str] = ()
    failureInfo: List[MarkOTErrorInfo] = ()
    serviceData: ServiceData = None


@dataclass
class AddMarkOTResponse(TcBaseObj):
    """
    AddMarkOTResponse structure defines the response from addMarkOTForCurrentUser operation. It contains the success
    and failure information of every objects for marks. It provides the errors if the operation fails too.
    
    :var successInfo: The list of the success information. It includes:
    1.    Business object UID.
    2.    Target site id.
    3.    Lock user id.
    4.    The anchor business object. It is used by ItemRevision objects only and the value is its Item. The value is
    empty for other objects.
    :var failureInfo: The list of failure information. It includes:
    1.    Business object UID.
    2.    Error code.
    3.    Error message.
    :var serviceData: Service data contains the errors if the operation fails.
    """
    successInfo: List[MarkOTInfo] = ()
    failureInfo: List[MarkOTErrorInfo] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class MarkupResponse(TcBaseObj):
    """
    This common MarkupResponse structure is for the response of both loadMarkups and saveMarkups operations.
    
    :var version: The version of the markups set by the server, opaque to the client.
    :var message: The message for client/server communication about the markups, excluding error messages.
    :var markups: The loaded or updated markups.
    :var serviceData: The Service Data that may contain partial errors.
    """
    version: str = ''
    message: str = ''
    markups: str = ''
    serviceData: ServiceData = None



from tcsoa.gen.Internal_VendorManagementAW._2019_12.services import VendorManagementService as imp0
from tcsoa.gen.Internal_VendorManagementAW._2020_05.services import VendorManagementService as imp1
from tcsoa.gen.Internal_VendorManagementAW._2020_12.services import VendorManagementService as imp2
from tcsoa.base import TcService


class VendorManagementService(TcService):
    assignPartnerContractToObject = imp0.assignPartnerContractToObject
    assignPartnerContractToObject2 = imp1.assignPartnerContractToObject2
    getAssignedProductsOfContract = imp1.getAssignedProductsOfContract
    getColumnInfo = imp0.getColumnInfo
    getPartnerContractsOfSelectedObject = imp0.getPartnerContractsOfSelectedObject
    removePartnerContractFromObject = imp0.removePartnerContractFromObject
    removePartnerContractFromObject2 = imp1.removePartnerContractFromObject2
    sortAndFilterViewModelRows = imp2.sortAndFilterViewModelRows



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, Item, ReleaseStatus, ItemRevision
from typing import List
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ReleaseStatusEffectivityInput(TcBaseObj):
    """
    The information required to create effectivity on a release status
    
    :var releaseStatus: The release status for which to set effectivity
    :var effectivityInfoInput: A structure to hold effectivity input info
    """
    releaseStatus: ReleaseStatus = None
    effectivityInfoInput: EffectivityInfoInput = None


@dataclass
class CreateOccEffectivityInput(TcBaseObj):
    """
    The information required to create occurrence effectivity for a list of BOMLine 
    
    :var bomLines: The BOMLine objects for which to add effectivity
    :var effectivityInfoInput: A structure to hold effectivity info
    :var isShared: True to share effectivity among BOMLine objects
    False to not share effectivity among BOMLine objects
    """
    bomLines: List[BOMLine] = ()
    effectivityInfoInput: EffectivityInfoInput = None
    isShared: bool = False


@dataclass
class EffectivityInfoInput(TcBaseObj):
    """
    A structure to hold effectivity info
    
    :var effectivityId: Effectivity ID
    :var endItem: Effectivity end Item
    :var endItemRev: Effectivity end ItemRevision
    :var unitRangeText: Effectivity unit range, a valid range of unit numbers. Always specified in the context of the
    end Item to which the units apply. It can be a discrete,noncontinuous range.
    :var dateRange: The array of effectivity date range,a valid range of dates. 
    -    Open range,for example,from 05 January onward.
    -    Closed range,for example,from 01 January to 30 April.
    
    :var openEndedStatus: Effectivity open ended status, 0 for EFFECTIVITY_closed, 1 for EFFECTIVITY_open_ended, 2 for
    FFECTIVITY_stock_out
    :var isProtected: True to add new range to any existing ranges
    False to overwrite existing ranges
    """
    effectivityId: str = ''
    endItem: Item = None
    endItemRev: ItemRevision = None
    unitRangeText: str = ''
    dateRange: List[datetime] = ()
    openEndedStatus: int = 0
    isProtected: bool = False



from __future__ import annotations

from tcsoa.gen.StructureManagement._2008_05.StructureSearch import OccurrenceNoteExpression, SearchBySizeExpression, FormAttributeExpression, BoxZoneExpression, InClassExpression, SavedQueryExpression, AttributeExpression
from tcsoa.gen.BusinessObjects import BOMWindow, BusinessObject, RuntimeBusinessObject
from typing import List
from tcsoa.gen.StructureManagement._2010_04.StructureSearch import PlaneZoneExpression
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ProximityExpression(TcBaseObj):
    """
    ProximityExpression specifies one set of proximity criteria. The search results for each lines are combined
    together using an 'OR' operation.
    
    :var srcLines: A list of source Fnd0BOMLineLite or BOMLine objects to be used in the proximity search.
    :var distance: Proximity distance in metres from the outer surface of the Fnd0BOMLineLite or BOMLine geometry.
    :var includeChildLines: If true, all child Fnd0BOMLineLite objects or BOMLine of source srcLines are considered in
    the proximity search.
    """
    srcLines: List[RuntimeBusinessObject] = ()
    distance: float = 0.0
    includeChildLines: bool = False


@dataclass
class SearchExpressionSet(TcBaseObj):
    """
    The full set of search expressions that are to be used to perform a single structure search. Each of the
    expressions provided are combined using a logical 'AND' operator.
    
    :var itemAndRevisionAttributeExpressions: A list of Item and ItemRevision attribute search expressions.
    :var occurrenceNoteExpressions: A list of Occurrence Note attribute search expressions.
    :var returnScopedSubTreesHit: If true, return only the subset of 'scopeBomLines' provided in input as part of
    'SearchScope'.
    :var executeVOOFilter: If true, Valid Overlays Only (VOO) filter is applied prior to returning the results from the
    server.
    :var executeRemoteSearch: If true, the search includes results from local and remote site(s) in response otherwise
    search results from local site are returned.
    :var remoteSiteID: The remote site ID to search from. By default, if the site ID is not specified, the search will
    use the site id that is configured by the admin in Teamcenter.
    :var formAttributeExpressions: A list of Form attribute search expressions.
    :var proximitySearchExpressions: A list of spatial proximity search expressions.
    :var boxZoneExpressions: A list of spatial box zone expressions.
    :var planeZoneExpressions: A list of spatial plane zone search expressions.
    :var savedQueryExpressions: A list of saved query search expressions.
    :var inClassQueryExpressions: A list of class attribute search expressions.
    :var sizeSearchExpression: A list of spatial size search expressions.
    :var doTrushapeRefinement: If true, Trueshape is perform. (use actual geometry rather using bounding box covering
    min and max points) refinement on the spatial search.
    """
    itemAndRevisionAttributeExpressions: List[AttributeExpression] = ()
    occurrenceNoteExpressions: List[OccurrenceNoteExpression] = ()
    returnScopedSubTreesHit: bool = False
    executeVOOFilter: bool = False
    executeRemoteSearch: bool = False
    remoteSiteID: str = ''
    formAttributeExpressions: List[FormAttributeExpression] = ()
    proximitySearchExpressions: List[ProximityExpression] = ()
    boxZoneExpressions: List[BoxZoneExpression] = ()
    planeZoneExpressions: List[PlaneZoneExpression] = ()
    savedQueryExpressions: List[SavedQueryExpression] = ()
    inClassQueryExpressions: List[InClassExpression] = ()
    sizeSearchExpression: SearchBySizeExpression = None
    doTrushapeRefinement: bool = False


@dataclass
class SearchScope(TcBaseObj):
    """
    SearchScope specifies the scope of a search.
    
    :var window: The BOMWindow object specifying a structure context within which to perform search.
    :var scopeLines: A list of Fnd0BOMLineLite or BOMLine objects, specifying scope of the search.
    :var ignoreOccurrenceTypes: A list of occurrence types that are to be ignored from search results.
    """
    window: BOMWindow = None
    scopeLines: List[RuntimeBusinessObject] = ()
    ignoreOccurrenceTypes: List[str] = ()


@dataclass
class StructureSearchResultResponse(TcBaseObj):
    """
    The result set received from a single step in a structure search.
    
    :var serviceData: An object of 'ServiceData' which contains any exceptions if occurred during performing search.
    :var outLines: A batch of Fnd0BOMLineLite or BOMLine objects that satisfy the search criteria for this search.
    :var linesDone: The number of lines returned in the response.
    :var searchCursor: Search cursor object that holds the current state of the search in the BOM session.
    :var finished: If true, the search is complete; otherwise, call 'nextSearch2' to get next set of results.
    """
    serviceData: ServiceData = None
    outLines: List[RuntimeBusinessObject] = ()
    linesDone: int = 0
    searchCursor: BusinessObject = None
    finished: bool = False



from __future__ import annotations

from tcsoa.gen.Internal.Core._2012_09.Envelope import EmailInfo
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class EnvelopeService(TcService):

    @classmethod
    def sendEmail(cls, emailinfos: List[EmailInfo]) -> ServiceData:
        """
        This operation sends email for each EmailInfo supplied. EmailInfo structure contains the email information such
        as the subject, message body, list of external and internal Teamcenter recipients, and FMS read tickets for the
        file attachments. File attachments are retrieved from FMS using the given FMS read tickets.
        """
        return cls.execute_soa_method(
            method_name='sendEmail',
            library='Internal-Core',
            service_date='2012_09',
            service_name='Envelope',
            params={'emailinfos': emailinfos},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Internal.AuditManager._2012_09.ProcessHistoryManagement import GetProcessHistoryAuditRecordsResponse
from tcsoa.base import TcService
from tcsoa.gen.BusinessObjects import WorkspaceObject


class ProcessHistoryManagementService(TcService):

    @classmethod
    def getProcssHistoryAuditRecords(cls, wsoObject: WorkspaceObject) -> GetProcessHistoryAuditRecordsResponse:
        """
        Get process history audit records of given workspace object. One process usally contains multiple history audit
        records and one given workspace object may have multiply processes.
        """
        return cls.execute_soa_method(
            method_name='getProcssHistoryAuditRecords',
            library='Internal-AuditManager',
            service_date='2012_09',
            service_name='ProcessHistoryManagement',
            params={'wsoObject': wsoObject},
            response_cls=GetProcessHistoryAuditRecordsResponse,
        )



from __future__ import annotations

from tcsoa.gen.UiConfig._2014_11.UiConfig import GetUIConfigInput
from tcsoa.gen.UiConfig._2015_10.UiConfig import GetUIConfigResponse
from typing import List
from tcsoa.base import TcService


class UiConfigService(TcService):

    @classmethod
    def getUIConfigs2(cls, getUiConfigsIn: List[GetUIConfigInput]) -> GetUIConfigResponse:
        """
        This operation returns information used by the client to render the User Interface. The information returned
        includes command and column configuration information. This operation replaces getUIConfigs operation, it
        returns the Fnd0CommandCollectionRel objects that associate the top level command collections to client scope
        in CommandConfigData structure in addition to Fnd0CommandCollection objects.
        """
        return cls.execute_soa_method(
            method_name='getUIConfigs2',
            library='UiConfig',
            service_date='2015_10',
            service_name='UiConfig',
            params={'getUiConfigsIn': getUiConfigsIn},
            response_cls=GetUIConfigResponse,
        )



from __future__ import annotations

from tcsoa.gen.StructureManagement._2016_09.PublishByLink import FindSourcesResponse
from tcsoa.gen.StructureManagement._2007_06.PublishByLink import LineAndWindow
from typing import List
from tcsoa.base import TcService


class PublishByLinkService(TcService):

    @classmethod
    def findSourcesInWindow(cls, input: List[LineAndWindow]) -> FindSourcesResponse:
        """
        Finds all source BOMLines objects of the PublishLink in source BOMWindow for input target BOMLine objects. All
        sources are returned as BOMLine objects.
        
        Use cases:
        Determine if BOMWindow has sources for input target BOMLine objects.
        """
        return cls.execute_soa_method(
            method_name='findSourcesInWindow',
            library='StructureManagement',
            service_date='2016_09',
            service_name='PublishByLink',
            params={'input': input},
            response_cls=FindSourcesResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetBriefcaePreviewDataResponse(TcBaseObj):
    """
    GetBriefcaePreviewDataResponse structure defines the response from GetBriefcaePreviewData operation. It contains
    FMS tickets or preview data of the old briefcase file, the new briefcase file and the delta briefcase file. It
    provides the errors if SOA fails too.
    
    :var oldBriefcasePreviewDataFMSTicket: FMS ticket of the preview data from the briefcase specified in
    oldBriefcaseFMSTicket or oldBriefcaseUID which can be used to download from server to client. The preview data
    include the structure in the briefcase and can be displayed on the client.
    :var newBriefcasePreviewDataFMSTIcket: FMS ticket of the preview data from the briefcase specified in
    newBriefcaseFMSTicket or newBriefcaseUID which can be used to download from server to client. The preview data
    include the structure in the briefcase and can be displayed on the client.
    :var deltaBriefcasePreviewFMSTicket: FMS ticket of the delta briefcase preview data from the two input briefcases
    specified in (oldBriefcaseFMSTicket or oldBriefcaseUID) and (newBriefcaseFMSTicket or newBriefcaseUID)   which can
    be used to download from server to client. The delta preview data include the difference of the structures between
    the two input briefcases and can be displayed on the client.
    :var oldBriefcasePreviewData: The preview data from the briefcase specified in oldBriefcaseFMSTicket or
    oldBriefcaseUID. The preview data include the structure in the briefcase and can be displayed on the client.
    :var newBriefcasePreviewData: The preview data from the briefcase specified in newBriefcaseFMSTicket or
    newBriefcaseUID. The preview data include the structure in the briefcase and can be displayed on the client.
    :var deltaBriefcasePreviewData: The delta preview data from the two input briefcases specified in
    (oldBriefcaseFMSTicket or oldBriefcaseUID) and (newBriefcaseFMSTicket or newBriefcaseUID). The delta preview data
    include the difference of the structures between the two input briefcases and can be displayed on the client.
    :var serviceData: Service data contains the errors if the operation fails.
    """
    oldBriefcasePreviewDataFMSTicket: str = ''
    newBriefcasePreviewDataFMSTIcket: str = ''
    deltaBriefcasePreviewFMSTicket: str = ''
    oldBriefcasePreviewData: List[NamesAndValues] = ()
    newBriefcasePreviewData: List[NamesAndValues] = ()
    deltaBriefcasePreviewData: List[NamesAndValues] = ()
    serviceData: ServiceData = None


@dataclass
class NamesAndValues(TcBaseObj):
    """
    NamesAndValues structure represents a generic name-value pair
    
    :var elementName: The name of the name-value pair
    :var elementValues: The values of the name-values pair.
    """
    elementName: str = ''
    elementValues: List[str] = ()


@dataclass
class CheckBriefcaseLicenseResponse(TcBaseObj):
    """
    Response for Briefcase license check operation. Contains true if multisite_server and tc_briefcase license are
    available.
    
    :var licenseExists: True if both "mulitsite_server" and "tc_briefcase" licenses are available.
    :var serviceData: This contains the status of the operation.
    """
    licenseExists: bool = False
    serviceData: ServiceData = None



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class RetrieveQueryCriteriaResponse(TcBaseObj):
    """
    RetrieveQueryCriteriaResponse
    
    :var output: vector of SaveQueryCriteriaInfo
    :var serviceData: standard ServiceData member
    """
    output: List[SaveQueryCriteriaInfo] = ()
    serviceData: ServiceData = None


@dataclass
class SaveQueryCriteriaInfo(TcBaseObj):
    """
    SaveQueryCriteriaInfo
    
    :var searchName: The name of the saved search.
    :var queryName: The name of the query associated with this search.
    :var keys: The attribute names for the query.
    :var values: The attribute values for the query.
    """
    searchName: str = ''
    queryName: str = ''
    keys: List[str] = ()
    values: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import WorkspaceObject, Cfg0ConfiguratorPerspective
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class ChildNode(TcBaseObj):
    """
    ChildNode is a ViewModelObject that represents the child for a given parent object. It encapsulates a model object
    in server side. This object can represent configurator objects like Cfg0AbsFeature, Cfg0AbsFeatureFamily,
    Cfg0AbsProductLine, and Cfg0AbsProductModel.
    
    :var parentNode: Parent node in tree for this child node.
    :var node: The ViewModelObject to be rendered.
    """
    parentNode: ViewModelObjectNode = None
    node: ViewModelObjectNode = None


@dataclass
class GetConfiguratorDataHierarchyIn(TcBaseObj):
    """
    The structure containing parameters to fetch the children for given parent Business Object.
    
    :var configPerspective: The configurator perspective. When this operation is invoked first time, this parameter
    would be null. First call returns this parameter and caller must hold to that parameter for passing through
    subsequent calls.
    :var parent: The parent object for which configurator data to be retrieved
    :var sortCriteria: Criteria to be used to sort the filtered results.
    :var requestInfo: Map (string/string) of configuration parameters.
    
    Allowed key | values are: 
    
    "RequestType" key provides information about the data that needs to be displayed to be handled for this service
    operation call.
    "RequestType" | "Products", product Line Hierarchy is displayed. "RequestType" | "Features", variability data is
    displayed. 
    
    "Expand" key provides the information of level of expansion for a given object.   
    "Expand" | "0", complete hierarchy is displayed. 
    "Expand" | "1", only one level of hierarchy is displayed. 
    Default value of "Expand" is "0". 
    
    "View" key provides information about the type of view. 
    "View" | "Tree", information is provided in hierarchical format. 
    Default value of "View" is "Tree". Key and Value in map are case sensitive. This parameter can be used in future to
    support different types of Views e.g. "List".
    
    "configSettings" key stores configuration information in JSON string which will be used to retrieve content
    "configSettings" JSON string supports following keys:
    "pca0RevisionRule": contains the Revision Rule name based on recent configurator perspective settings.
    "pca0Effectivity": contains the Effectivity expression string based on recent configurator perspective settings.
    "pca0RuleDate": contains the Rule Date based on recent configurator perspective settings.
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    parent: WorkspaceObject = None
    sortCriteria: SortCriteria = None
    requestInfo: StringMap2 = None


@dataclass
class GetConfiguratorDataHierarchyResp(TcBaseObj):
    """
    The output containing the available variability for the input list of variant option vaue selections.
    
    :var configPerspective: The configurator perspective containing all information about the current Configurator
    Context, revision rule and effectivity. All further communications with the server to retrieve variant
    configuration data must use this object.
    :var parent: The parent object for which configurator data to be retrieved.
    :var childrenNode: Teamcenter WorkspaceObject that was searched for and to be rendered in the view.
    :var serviceData: Contains the list of errors that might have occurred as part of the service invocation.
    :var responseInfo: Map (string/string) of configuration parameters.
    
    Keys supported: 
    "configSettings": contains the configuration settings for perspective object. This is JSON string.
    "configSettings" JSON string supports following keys:
    "pca0RevisionRule": contains the updated Revision Rule string based on recent configurator perspective object.
    "pca0Effectivity": contains Effectivity expression string based on recent configurator perspective object.
    "pca0RuleDate": contains Rule Date based on recent configurator perspective object.
    
    "pca0ConfigPerspective": contains latest updated Configurator Perspective properties.
    """
    configPerspective: Cfg0ConfiguratorPerspective = None
    parent: WorkspaceObject = None
    childrenNode: List[ChildNode] = ()
    serviceData: ServiceData = None
    responseInfo: StringMap2 = None


@dataclass
class SortCriteria(TcBaseObj):
    """
    Specifies the criteria to use to sort the results that are retrieved. It provides the field to sort and the
    direction to sort it in.
    
    :var fieldName: The name of the field to perform the sorting. This has to be the name of a property of an
    Configurator object on which sorting to be performed.
    :var sortDirection: The direction in which the sorting needs to be perfomed. It could be ascending or descending.
    Valid values are: "ASC" and "DESC".
    """
    fieldName: str = ''
    sortDirection: str = ''


@dataclass
class ViewModelObjectNode(TcBaseObj):
    """
    Service operation output structure to store the Teamcenter WorkspaceObject and its associated View model properties
    to be displayed in the view.
    
    :var wsObject: WorkspaceObject representing the node.
    :var nodeProperties: A list of ViewModelObjectProperty for the node.
    """
    wsObject: WorkspaceObject = None
    nodeProperties: List[ViewModelObjectProperty] = ()


@dataclass
class ViewModelObjectProperty(TcBaseObj):
    """
    Properties of the viewmodel object.
    
    :var propertyName: Property name.
    :var nodePropertyInfo: Map of (string, string) child node property and value.
    Valid keys: "isArray", "isEnabled", "isEditable"
    Valid Values: "true", "false".
    :var values: Real value for the property, the values are converted to strings for other value types such as date,
    double, Boolean or referenced objects.
    :var displayValues: Display values for the property.
    :var srcobjLsd: Last saved date of source object.
    """
    propertyName: str = ''
    nodePropertyInfo: PropertyInfoMap = None
    values: List[str] = ()
    displayValues: List[str] = ()
    srcobjLsd: datetime = None


"""
Map of string and list of strings.
"""
PropertyInfoMap = Dict[str, str]


"""
Map of string key and string as value.
"""
StringMap2 = Dict[str, str]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine, ItemRevision
from tcsoa.gen.Cad._2007_12.StructureManagement import DeleteRelativeStructurePref2, CreateOrUpdateAbsoluteStructureInfo2, DeleteRelativeStructureInfo2, CreateOrUpdateRelativeStructureInfo2, CreateOrUpdateRelativeStructurePref2, ClassicOptionsResponse, VariantConditionResponse, CreateOrUpdateAbsoluteStructurePref2, DeleteAssemblyArrangementsInfo2, DeleteAssemblyArrangementsPref
from tcsoa.gen.Cad._2007_01.StructureManagement import DeleteRelativeStructureResponse, CreateOrUpdateRelativeStructureResponse, CreateOrUpdateAbsoluteStructureResponse, DeleteAssemblyArrangementsResponse
from tcsoa.gen.Cad._2007_12.DataManagement import PartInfo2, CreateOrUpdatePartsPref
from tcsoa.gen.Cad._2007_01.DataManagement import CreateOrUpdatePartsResponse
from typing import List
from tcsoa.base import TcService


class StructureManagementService(TcService):

    @classmethod
    def queryClassicOptions(cls, inputRevisions: List[ItemRevision]) -> ClassicOptionsResponse:
        """
        This operation is to find classic variant options defined on an ItemRevision object(s).
        
        Use cases:
        The user can use this operation to find all the classic variant options defined on given ItemRevision object.
        The option and values can be used for configuration or creating other related objects like variant conditions,
        constraint (defaults, derived defaults and rule checks) rules or VariantRule objects.
        """
        return cls.execute_soa_method(
            method_name='queryClassicOptions',
            library='Cad',
            service_date='2007_12',
            service_name='StructureManagement',
            params={'inputRevisions': inputRevisions},
            response_cls=ClassicOptionsResponse,
        )

    @classmethod
    def queryVariantConditions(cls, inputBomLines: List[BOMLine]) -> VariantConditionResponse:
        """
        This operation is to find a variant condition value ( load if - this is a type of variant expression represents
        variant condition) for a BOMLine object.
        
        Use cases:
        The user needs variant condition defined on the BOMLine object, for display or editing purpose.
        """
        return cls.execute_soa_method(
            method_name='queryVariantConditions',
            library='Cad',
            service_date='2007_12',
            service_name='StructureManagement',
            params={'inputBomLines': inputBomLines},
            response_cls=VariantConditionResponse,
        )

    @classmethod
    def createOrUpdateAbsoluteStructure(cls, info: List[CreateOrUpdateAbsoluteStructureInfo2], bomViewTypeName: str, complete: bool, pref: CreateOrUpdateAbsoluteStructurePref2) -> CreateOrUpdateAbsoluteStructureResponse:
        """
        This is the overloaded function of Cad::_2007_01::StructureManagement::createOrUpdateAbsoluteStructure.
        It takes an additional variable into the CreateOrUpdateAbsoluteStructurePref2 structure to hold the last
        modified date of the BVR with an extra preference value to check it with server modifed date to make a decision
        whether we want to make modification on BVR.
        CreateOrUpdateAbsoluteStucture allows the user to find or create the absolute structure network of objects and
        relations for the input model.
        The service first attempts to check for the presence of duplicate context objects and then validate the
        existence of the structure to be created.
        If any of the objects exist and the input attribute values differ from those already set, an attempt is made to
        update the values.
        Note: The following AbsOccData attributes are not supported for arrangement qualified overrides.
        These attributes can only be overridden at the bvr level (which applies to all arrangements).
        If these attributes are overridden in the AssemblyArrangementInfo, they will be ignored.
        1.child item
        2.GDE object
        3.instance number
        4.occurrence name
        5.note
        6.occurrence type
        7.Variant instance
        As we process one contextItemRev object at one time, it is assumed that all edits for a given contextItemRev
        come in as one set of input.
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in bomViewTypeName. 
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateAbsoluteStructure',
            library='Cad',
            service_date='2007_12',
            service_name='StructureManagement',
            params={'info': info, 'bomViewTypeName': bomViewTypeName, 'complete': complete, 'pref': pref},
            response_cls=CreateOrUpdateAbsoluteStructureResponse,
        )

    @classmethod
    def createOrUpdateRelativeStructure(cls, inputs: List[CreateOrUpdateRelativeStructureInfo2], bomViewTypeName: str, complete: bool, pref: CreateOrUpdateRelativeStructurePref2) -> CreateOrUpdateRelativeStructureResponse:
        """
        This is the overloaded function of Cad::_2007_01::StructureManagement::createOrUpdateRelativeStructure.
        It takes an additional variable into the CreateOrUpdateRelativeStructureInfo2 structure to hold the last
        modified date of the BVR with an extra preference value to check it with server modifed date to make a decision
        whether we want to make modification on BVR.
        Create or update the relative structure objects and relations for the input model.
        The service first attempts to check for the presence of duplicate context objects and then validate the
        existence of the structure to be created.
        If any of the objects exist but the input attribute values that differ from those already set, an attempt is
        made to update the values if possible.
        This service assumes the input is in a bottom-up format such that if any failures occur, the structure that is
        created will be consistent.
        No attempt is made in the service to rearrange the input and process it in the correct order.
        As we process one parent context Object at one time, it is assumed that all edits for a given parent context
        come in as one set of input.
        
        Exceptions:
        >Service exceptions are thrown for the following error situations:
        
        215005: The BOM view revision (BVR) is not checked out by the current user.
        215006: The BOM view revision is checked out by another user.
        215009: The object type name specified in the input preference does not exist.
        215010: The input CAD occurrence identifier does not correspond to a valid BOM line. This is an error when the
        complete input is false.
        215033: The input last modified date differs from actual.
        215034: The dataset or BVR was modified even when the input last modified dates was different than the current
        last modified data. 
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateRelativeStructure',
            library='Cad',
            service_date='2007_12',
            service_name='StructureManagement',
            params={'inputs': inputs, 'bomViewTypeName': bomViewTypeName, 'complete': complete, 'pref': pref},
            response_cls=CreateOrUpdateRelativeStructureResponse,
        )

    @classmethod
    def deleteAssemblyArrangements(cls, info: List[DeleteAssemblyArrangementsInfo2], bomViewTypeName: str, pref: DeleteAssemblyArrangementsPref) -> DeleteAssemblyArrangementsResponse:
        """
        This operation deletes assembly arrangements and all the absolute occurrence data associated with the assembly
        arrangements.  It also deletes the relation between assembly arrangements and the BOM view revision object. The
        last modified date of the BVR can be specified for comparison against the last modified date in Teamcenter to
        ensure the BVR that contains the arrangement has not changed outside of this client context.
        
        Use cases:
        User deletes an existing assembly arrangement from an existing assembly.
        
        For this operation, the assembly is passed in as the 'itemRev' and the assembly arrangement is passed in
        through 'arrangements'.  The assembly arrangement is removed from the assembly, as well as the relation between
        the arrangement and the BVR, and the UID of the deleted arrangement is added to the ServiceData list of deleted
        objects.
        
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in 'bomViewTypeName'.
        """
        return cls.execute_soa_method(
            method_name='deleteAssemblyArrangements',
            library='Cad',
            service_date='2007_12',
            service_name='StructureManagement',
            params={'info': info, 'bomViewTypeName': bomViewTypeName, 'pref': pref},
            response_cls=DeleteAssemblyArrangementsResponse,
        )

    @classmethod
    def deleteRelativeStructure(cls, inputs: List[DeleteRelativeStructureInfo2], bomViewTypeName: str, pref: DeleteRelativeStructurePref2) -> DeleteRelativeStructureResponse:
        """
        Deletes one or more first level children of a parent assembly.  This is the overloaded function of
        Cad::_2007_01::StructureManagement::deleteRelativeStructure.
        It takes an additional variable into the DeleteRelativeStructureInfo2 structure to hold the last modified date
        of the BVR with an extra preference value to check it with server modifed date to make a decision whether we
        want to make modification on BVR.
        
        Exceptions:
        >Service Exception    Thrown if an invalid BOM view type is specified in bomViewTypeName. 
        """
        return cls.execute_soa_method(
            method_name='deleteRelativeStructure',
            library='Cad',
            service_date='2007_12',
            service_name='StructureManagement',
            params={'inputs': inputs, 'bomViewTypeName': bomViewTypeName, 'pref': pref},
            response_cls=DeleteRelativeStructureResponse,
        )


class DataManagementService(TcService):

    @classmethod
    def createOrUpdateParts(cls, info: List[PartInfo2], pref: CreateOrUpdatePartsPref) -> CreateOrUpdatePartsResponse:
        """
        CreateOrUpdateParts allows the user to create or update a set of parts using Items, Item Revisions, Datasets
        and ExtraObjects and also changes the ownership of the newly created object to the user/group supplied.
        If the user supplies a valid Item object reference without specifying a valid item revision object reference or
        id, then only the item will be updated.
        If the user specifies a valid item object reference with a null item revision object reference and a non-null
        revision id, then a new item revision will be created and attached to the item with no relations from the new
        item revision to the previous item revision.
        If the user specifies a valid item object reference and a valid item revision object reference, then the item
        and item revision and related objects will be updated.
        If no item object reference or item revision object references are specified then a new item and item revision
        and related objects will be created.
        All objects created and updated will be returned in the ServiceData.
        All partial errors will contain the clientIDs for all items related to the error message, i.e. if a dataset
        encounters an error, then the ID for that erro will be the item client ID concantentated with the item revision
        id contantenated with the dataset client ID, all separated by semi-colons ( ; ).
        """
        return cls.execute_soa_method(
            method_name='createOrUpdateParts',
            library='Cad',
            service_date='2007_12',
            service_name='DataManagement',
            params={'info': info, 'pref': pref},
            response_cls=CreateOrUpdatePartsResponse,
        )



from __future__ import annotations

from tcsoa.gen.Core._2008_06.DataManagement import CreateInput
from tcsoa.gen.BusinessObjects import BusinessObject, WorkspaceObject, Folder
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class BulkCreIn(TcBaseObj):
    """
    Input for bulk create operation including unique client identifier.
    
    :var clientId: client id Unique client identifier.
    :var quantity: Number of instances of each type for creation.For Asp0Aspect objects,'CreateInput' needs to be
    populated with the number of Asp0Aspect objects to be created hence quantity should be passed as 1.
    :var folder: Containing folder for the newly created items.
    :var data: Input data for create operation.
    """
    clientId: str = ''
    quantity: int = 0
    folder: Folder = None
    data: CreateInput = None


@dataclass
class ValidationResponse(TcBaseObj):
    """
    Return structure for 'createConnections' operation
    
    :var validationResult: A map of type <'std::string ,bool'> indicating if the ID is valid. The key is clientId from
    'CreateIn'.  The value is true if the id is valid, otherwise id is invalid.
    :var serviceData: Service data
    """
    validationResult: ValidationResult = None
    serviceData: ServiceData = None


@dataclass
class WhereUsedConfigParameters(TcBaseObj):
    """
    Additional Configuration Parameters required for 'whereUsed' search. For example 'tagMap' to specify the
    RevisionRule with revision_rule as a key, 'boolMap' to specify 'whereUsedPreciseFlag' with whereUsedPreciseFlag as
    a key, and 'intMap' to specify number of levels with numLevels as a key.
    
    :var stringMap: String Input parameters ( std::string, std::string )
    :var doubleMap: Double Input parameters ( std::string, double )
    :var intMap: int Input parameters ( std::string, int )
    :var boolMap: bool Input parameters ( std::string, bool )
    :var dateMap: Date Input parameters ( std::string, Teamcenter::DateTime )
    :var tagMap: Object Input parameters   ( std::string, BusinessObjectRef ( Teamcenter::BusinessObject ) )
    :var floatMap: Float Input Parameters ( std::string, float )
    """
    stringMap: WUStringMap = None
    doubleMap: WUDoubleMap = None
    intMap: WUIntMap = None
    boolMap: WUBoolMap = None
    dateMap: WUDateMap = None
    tagMap: WUTagMap = None
    floatMap: WUFloatMap = None


@dataclass
class WhereUsedInputData(TcBaseObj):
    """
    Input data for 'whereUsed' search contains the target object to perform search along with the configuration
    parameters.
    
    :var inputObject: Target object to do 'whereUsed' search
    :var useLocalParams: Flag to decide which config paramerters to be used, local or common
    :var inputParams: Additional Configuration Parameters required for 'whereUsed' search
    :var clientId: Client ID to uniquely idnetify this input data
    """
    inputObject: WorkspaceObject = None
    useLocalParams: bool = False
    inputParams: WhereUsedConfigParameters = None
    clientId: str = ''


@dataclass
class WhereUsedOutputData(TcBaseObj):
    """
    This structure contains output information of Where Used Call.
    
    :var inputObject: Input object
    :var info: List of where used result objects
    :var clientId: Client ID of input object which this output object is for
    """
    inputObject: WorkspaceObject = None
    info: List[WhereUsedParentInfo] = ()
    clientId: str = ''


@dataclass
class WhereUsedParentInfo(TcBaseObj):
    """
    Generic Parent Info Structure
    
    :var parentObject: Parent Workspace Object
    :var level: levet at which parent is found
    """
    parentObject: WorkspaceObject = None
    level: int = 0


@dataclass
class WhereUsedResponse(TcBaseObj):
    """
    'WhereUsedResponse' object contains list of  'WhereUsedOutputData' structure. This structure contains list of
    ItemRevision objects which are result of 'whereUsed' search.
    
    :var output: List of 'WhereUsedOutputData' structures
    :var serviceData: Standard 'ServiceData' Member
    """
    output: List[WhereUsedOutputData] = ()
    serviceData: ServiceData = None


"""
A map of type bool indicating if the ID is valid. The key is clientId from 'CreateIn'. The value is of type bool.  True if id is valid, otherise id is invalid
"""
ValidationResult = Dict[str, bool]


"""
Map of bool property names to values ('string, bool').
"""
WUBoolMap = Dict[str, bool]


"""
Map of DateTime property names to values '(string, DateTime').
"""
WUDateMap = Dict[str, datetime]


"""
Map of double property names to values ('string, double').
"""
WUDoubleMap = Dict[str, float]


"""
Map of float property names to values ('string, float').
"""
WUFloatMap = Dict[str, float]


"""
Map of int property names to values ('string, in').
"""
WUIntMap = Dict[str, int]


"""
Map of sting property names to values '(string, string').
"""
WUStringMap = Dict[str, str]


"""
 Map of BusinessObject property names to values ('string, BusinessObject').
"""
WUTagMap = Dict[str, BusinessObject]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Cad._2008_06.StructureManagement import RelativeStructureChildInfo2
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class MappedReturnData(TcBaseObj):
    """
    Map of returned data that is relevant to the created or updated occurrence for the given client ID.
    
    :var occThread: The occurrence thread for tracking the created or updated occurrence.
    :var occurrence: The occurrence object that was created or updated.
    :var bvr: The BOM view revision that was created or updated.
    """
    occThread: BusinessObject = None
    occurrence: BusinessObject = None
    bvr: BusinessObject = None


@dataclass
class AttributesInfoForObject(TcBaseObj):
    """
    Contains classname and vector of AttributesInfos structure
    
    :var topLineAttrThatRefersToObject: Name of the attribute that refers to the object
    :var attrsToSet: List of AttributesInfos
    """
    topLineAttrThatRefersToObject: str = ''
    attrsToSet: List[AttributesInfos] = ()


@dataclass
class AttributesInfos(TcBaseObj):
    """
    Contains name/values pair data to be set as attributes on the related object.
    
    :var name: Attribute name
    :var values: Attribute values
    """
    name: str = ''
    values: List[str] = ()


@dataclass
class RelativeStructureParentInfo(TcBaseObj):
    """
    Contains information about the parent BOM line representation.
    
    :var clientId: A unique string supplied by the caller.  This ID is used to identify return data elements and
    partial errors associated with this input structure.  If 'clientId' is not provided then it can be difficult to
    align the input with any returned errors.
    :var complete: Flag that if true signifies that the structure represented in the input is the full representation
    of the structure under the input parent. Any other structure relations that exist in Teamcenter but are not
    represented in the input will be removed.
    :var bomViewTypeName: The type of the BOM view to create under the parent 'bomViewTypeName' object.  If not
    specified, 'CreateOrUpdateRelativeStructurePref' 'bomViewTypeName' will be used as the default.
    :var parent: Object reference of the Item Revision context assembly object to create or update the child
    occurrence.  This is a required input.  An error will be returned if a valid parent is not specified.
    :var changeContext: Designated for future implementation.
    :var lastModifiedOfBVR: Last modified date of BOM view revision (BVR) under the input 'parent'.  This input is not
    required.  If this input date is different than the current last modified date and the 'overwriteForLastModDate'
    preference is false the input will be ignored and processing will continue with the next input.  In this scenario,
    error 215033 will be returned.  If the dates are different and the 'overwriteForLastModDate' preference is true,
    processing will continue with the current input.  In this scenario, error 215034 will be returned.
    :var precise: Flag for updating the BVR to precise (true) or imprecise (false).  Specifying precise as true means
    the child occurrences reference item revisions, whereas specifying precise as false (imprecise) means the child
    occurrences reference items.
    :var supportingClassAttrs: A list of object property names for the parent BOM line and attributes to set on the
    corresponding objects.  For example, 'supportingClassAttrs' 'topLineAttrThatRefersToObject' could be set to the
    property name bl_bomview_rev and 'attrsToSet' could include the attribute legacy_transform_factor with a value of
    0.5 to set on the BOM view revision object.
    """
    clientId: str = ''
    complete: bool = False
    bomViewTypeName: str = ''
    parent: BusinessObject = None
    changeContext: BusinessObject = None
    lastModifiedOfBVR: datetime = None
    precise: bool = False
    supportingClassAttrs: List[AttributesInfoForObject] = ()


@dataclass
class CreateOrUpdateRelativeStructureInfo(TcBaseObj):
    """
    Input structure for 'createOrUpdateRelativeStructure'.
    
    :var parentInfo: Parent info structure containing information about the parent BOM line representation.
    :var childInfo: Child info structure for creating the occurrence or updating the occurrence attributes.  If no
    child objects are specified and 'RelativeStructureParentInfo' 'complete' is true, all existing child objects will
    be removed.  If no child objects are specified and 'RelativeStructureParentInfo' 'complete' is false, the input is
    ignored.
    """
    parentInfo: RelativeStructureParentInfo = None
    childInfo: List[RelativeStructureChildInfo2] = ()


@dataclass
class CreateOrUpdateRelativeStructurePref(TcBaseObj):
    """
    Preference structure for 'createOrUpdateRelativeStructure'.
    
    :var overwriteForLastModDate: Flag to check whether the structure needs to be updated if the input last modified
    date is different from the current last modified date of the object in Teamcenter.  If false, but the
    'RelativeStructureParentInfo' 'lastModifiedOfBVR' input specified is different than the set modified date, partial
    error 215033 will be returned.
    :var continueOnError: Flag to indicate whether the operation should continue processing to the next input when an
    error is encountered.
    :var bomViewTypeName: The default BOM view type to create or the view type of the child to be added to the parent
    BOMViewRevision This default value can be overridden for individual parent by specifying the 'bomViewTypeName' in 
    'RelativeStructureParentInfo' 'parentInfo' input. The default value can also be overridden for individual children
    by specifying the 'childBomViewType' in the 'RelativeStructureChildInfo' 'childInfo' input.
    :var cadOccIdAttrName: Represents the occurrence note type which holds the value for the CAD occurrence ID or this
    can be the PSOccurrenceThread UID if the integration does not use a note type.
    :var objectTypes: List of object types that the client is interested in.  If 'complete' is true, object types or
    subtypes that exist in the structure in Teamcenter but are not in this list are removed.  If 'complete' is true,
    but no 'objectTypes' are specified, then all objects types or subtypes are removed from the existing structure in
    Teamcenter.  If 'complete' is false, 'objectTypes' is ignored.
    """
    overwriteForLastModDate: bool = False
    continueOnError: bool = False
    bomViewTypeName: str = ''
    cadOccIdAttrName: str = ''
    objectTypes: List[str] = ()


@dataclass
class CreateOrUpdateRelativeStructureResponse(TcBaseObj):
    """
    The response from the 'createOrUpdateRelativeStructure' operation.
    
    :var output: Map of client IDs to 'MappedReturnData'.
    :var serviceData: The 'ServiceData'.  This operation will populate the 'ServiceData' with created occurrences, BOM
    views and BOM view revisions, updated occurrences and BOM view revisions, and any implicitly deleted occurrences.
    """
    output: ClientIdToOccurrenceMap = None
    serviceData: ServiceData = None


"""
Map that maps from client Id to MappedReturnData
"""
ClientIdToOccurrenceMap = Dict[str, MappedReturnData]



from __future__ import annotations

from tcsoa.gen.ActiveWorkspaceVis._2015_03.DataManagement import SessionInfo, UserAgentDataInfo, IdInfo, ServerInfo, LaunchInfoResponse
from typing import List
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def createLaunchInfo(cls, idInfos: List[IdInfo], serverInfo: ServerInfo, userDataAgentInfo: UserAgentDataInfo, sessionInfo: SessionInfo) -> LaunchInfoResponse:
        """
        This service operation is an extension to the service operation with the same name located in the
        DataManagement Interface of the Visualization service library. This extension allows the support of launching
        ActiveWorkspace specific objects such as Awb0Element and Awb0ProductContextInfo.
        
        This service generates a VVI information which is used to launch Teamcenter Visualization viewers with selected
        objects from Teamcenter and preserve a two way communication link between the viewer and the server.  This
        operation can return the VVI information as a string buffer or as a read file ticket to a vvi/vfz file in the
        FMS transient file volume. The "UseTransientVolume" option passed into the service via the Idinfo structure
        controls how the VVI launch information is returned.
        
        Obtaining the launch information as a string might be usefule to avoid setup and use of the FMS system directly
        by the calling client.  It is the responsibility of the client to determine how to use the returned string
        buffer.  For example, the vvi string buffer(s) can be written out as a vvi or vfz file on the client and passed
        to visualization, or the string buffer can be passed directly to embedded visualization if using the PLMVis
        toolkit.  
        
        If returning the launch information as a FMS transient file ticket then the operation requires the Teamcenter
        File Management System (FMS) to be installed (including FCC and transient volumes) in order to retrieve the VVI
        file from the transient file volume. When operating in this mode, the operation generates the launch file (VFZ
        or VVI), stores it in the FMS transient volume, and returns the FMS ticket. The client that initiated this
        operation is responsible for downloading the transient file (VVI or VFZ) from the transient volume to a local
        file system using the transient ticket. The transient (VVI or VFZ) file is consumed by the Teamcenter
        Visualization client. The viewer establishs a server connection and loads the object(s) specified in the VVI
        file.  Launch on multiple objects will generate a VFZ file (zip of all the vvi files) and transient ticket of
        VFZ file would be sent to client.
        
        NOTE: VVI and VFZ files are not intended to be persisted and should be generated with each launch to Teamcenter
        Visualization. For example, the VVI format is not guaranteed to be supported if the server or viewer is
        updated. VFZ files are used if more than one object is launched at a time, while VVI files are used for single
        objects.
        
        As with the previous createLaunchInfo service operation located in the Visualization library, this service
        supports launch on Teamcenter persistent objects like Dataset, Item, ItemRevision, BOMViewRevision, BOMView. It
        also supports launch of selected BOMLines of a configured structure from Structure Manager or BOPLines from
        Manufacturing Process Planner, but in this case the caller must first create a VisStructureContext object and
        make it the launched object. However with this operation you may also launch objects of type Awb0Element and
        Awb0ProductContextInfo. See description of IdInfo for details.
        
        Valid launch object types and behavior such as priority order can be configured with the Teamcenter Preferences
        VMU_Datasets, VMU_FileSearchOrder and VMU_RelationSearchOrder.
        
        Use cases:
        This operation supports the mechanism of visualizing Teamcenter specific objects in Teamcenter Visualization
        client.
        """
        return cls.execute_soa_method(
            method_name='createLaunchInfo',
            library='ActiveWorkspaceVis',
            service_date='2015_03',
            service_name='DataManagement',
            params={'idInfos': idInfos, 'serverInfo': serverInfo, 'userDataAgentInfo': userDataAgentInfo, 'sessionInfo': sessionInfo},
            response_cls=LaunchInfoResponse,
        )



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetFilteredProjectObjectsOutput(TcBaseObj):
    """
    The list of business objects, which are filtered based on the  given filter criteria for a given  project.
    
    :var projectID: The project ID.
    :var objectsInProject: The list of UIDs of business objects which are filtered based on the   given filter criteria.
    """
    projectID: str = ''
    objectsInProject: List[str] = ()


@dataclass
class GetFilteredProjectObjectsResponse(TcBaseObj):
    """
    The list of business objects, which are filtered based on the  given filter criteria for a given  project.
    
    :var filteredObjectsInProjects: List of filtered busisness objects for each of the requested projects.
    :var serviceData: Partial errors, if any.
    """
    filteredObjectsInProjects: List[GetFilteredProjectObjectsOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.BusinessObjects import ItemRevision
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class AttributeExchangeService(TcService):

    @classmethod
    def updateDocumentProperties(cls, inputObjects: List[ItemRevision], checkConfiguration: bool) -> ServiceData:
        """
        Update the document properties of the ItemRevision object&rsquo;s related Dataset named reference&rsquo;s file
        using Logical Object attribute exchange.
        
        Use cases:
        Update ItemRevision&rsquo;s related Dataset named reference&rsquo;s file:
        If there is Logical Object attribute exchange is configured on the related Dataset object, this operation can
        be invoked to perform the attributre exchange on the Dataset named reference&rsquo;s file.
        """
        return cls.execute_soa_method(
            method_name='updateDocumentProperties',
            library='DocumentManagement',
            service_date='2018_06',
            service_name='AttributeExchange',
            params={'inputObjects': inputObjects, 'checkConfiguration': checkConfiguration},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from datetime import datetime


@dataclass
class GetActivitiesComparisonDetailsResponse(TcBaseObj):
    """
    Return the list of activities detail elements and service data for partial errors.
    
    :var details: for each input equivalent set holds the details of activities
    :var serviceData: to capture partial errors
    """
    details: List[ActivitiesDetails] = ()
    serviceData: ServiceData = None


@dataclass
class PartialMatchCriteria2(TcBaseObj):
    """
    a structure to capture generic Partial Match criteria
    
    :var intMap: map of string to vector or integers.
    :var dblMap: map of string to vector of doubles.
    :var strMap: map of string to vector of strings.
    :var objMap: map of string to vector of objects.
    :var dateMap: map of string to vector of dates
    """
    intMap: StringToIntVectorMap2 = None
    dblMap: StringToDblVectorMap2 = None
    strMap: StringToStrVectorMap2 = None
    objMap: StringToObjVectorMap2 = None
    dateMap: StringToDateVectorMap2 = None


@dataclass
class ToolRequirementComparisonData(TcBaseObj):
    """
    Specifies a data of the tool requirements.
    The fieldName specifies the data it represents such as TR description, Classification search class etc. 
    The Values specifies the data of source and target tool requirement combined. The first few data of Values
    corresponds to the source tool requirement followed by the target tool requirement. The vector size of Values is
    equal to the summation of vector size of source and target tool requirements input vector.
    The flag represents whether the data are different.
    
    :var fieldName: Specifies the field of tool requirement such as NAME, DESCRIPTION, SEARCHCLASS and SEARCHATTRIBUTE.
    NAME represents the name of the tool requirement.
     DESCRIPTION represents the tool requirement description.
     SEARCHCLASS represents the search class of the tool requirement.
    SEARCHATTRIBUTE represents one of the search attribute of classification search.
    :var values: Specifies the data of source and target tool requirement for the given field. The first few elements
    correspond to the source tool requirement followed by target tool requirement.
    :var isDifferent: For a field, specifies the flag indicating whether the data of source and target tool requirement
    is same. For multiple source and target tool requirement, if any two data are different then the flag is set to
    true.
    """
    fieldName: str = ''
    values: List[str] = ()
    isDifferent: bool = False


@dataclass
class ToolRequirementComparisonResult(TcBaseObj):
    """
    Specifies the comparison result of source and target tool requirements.
    
    :var comparisonData: Specifies the comparison data of all the fields of tool requirement. 
    The first element of the vector specifies the name of the tool requirement and will have NAME as fieldName.
    The second element of the vector specifies the description of tool requirement and will have DESCRIPTION as
    fieldName.
    If the search criteria is defined, the third element of the vector specifies the search class of tool requirement
    and will have SEARCHCLASS fieldName.
    If the search criteria is defined then the subsequent elements specifies the search attributes of the tool
    requirement. The elements will have SEARCHaTTRIBUTE as fieldName. The vector may have multiple elements with this
    fieldName.
    :var serviceData: Service data will hold warnings and errors, if any.
    """
    comparisonData: List[ToolRequirementComparisonData] = ()
    serviceData: ServiceData = None


@dataclass
class ActivitiesDetails(TcBaseObj):
    """
    details of activities for one equivalent set
    
    :var index: position in the input equivalentObjects vector
    :var details: details of all the activity tree for each operation in the equivalentLines vector
    :var equivalentLines: all sources of equivalent set in sequence and then all targets in sequence
    """
    index: int = 0
    details: ActivityDetailsElement = None
    equivalentLines: List[BusinessObject] = ()


@dataclass
class ActivityDetailsElement(TcBaseObj):
    """
    recursive structure that represents one row in the activities tree (one activity across all the input lines)
    
    :var isDifferent: result of comparison of this activity
    :var children: size of the vector will match the number of children that this activity has
    :var activities: size of the vector will match the size of equivalentLines vector
    """
    isDifferent: bool = False
    children: List[ActivityDetailsElement] = ()
    activities: List[BusinessObject] = ()


@dataclass
class EquivalentLines2(TcBaseObj):
    """
    Lines from a Source Window and a Target Window that are equivalent. For example - having the same ID in Context or
    other criteria.
    
    :var eqvSrcLines: set of source bomlines that are equivalent based on some criteria like ID in context.
    :var eqvTargetLines: set of target bomlines (not the same window as source lines) that are equivalent in a manner
    consistent with the source lines
    """
    eqvSrcLines: List[BusinessObject] = ()
    eqvTargetLines: List[BusinessObject] = ()


"""
a map of string to vector of dates
"""
StringToDateVectorMap2 = Dict[str, List[datetime]]


"""
String to vector of doubles map.
"""
StringToDblVectorMap2 = Dict[str, List[float]]


"""
map of string to vector of integers.
"""
StringToIntVectorMap2 = Dict[str, List[int]]


"""
a map of string to vector of objects.
"""
StringToObjVectorMap2 = Dict[str, List[BusinessObject]]


"""
a map of String to vector of strings.
"""
StringToStrVectorMap2 = Dict[str, List[str]]



from __future__ import annotations

from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class ExportSearchResultsResponse(TcBaseObj):
    """
    This structure contains the file ticket of exported file. . 
    
    The following partial errors may be returned: 
    141401    Unable to export object "<Object name>" with UID "<UID>". 
    141402    selectedObjectUIDs list cannot be empty when exporting selected objects.
    141403    The column configuration must be specified to export search results.
    141404    An internal error has occurred which caused the object export operation to fail.
    141405    The search input criteria cannot contain empty properties when exporting all objects.
    141406    The supported application format is MSExcelX.
    
    :var serviceData: The ServiceData. If an error occurs then ServiceData will contain partial error such as error
    generating transient file read ticket due to a configuration issue at the server.
    :var transientFileReadTickets: A list of transient file read tickets for the exported file.
    """
    serviceData: ServiceData = None
    transientFileReadTickets: List[str] = ()



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import Dict, List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class CreateIn2(TcBaseObj):
    """
    This is input structure for create operation including unique client identifier.
    
    :var clientId: Unique client identifier
    :var createData: Input data for create operation
    :var dataToBeRelated: Additional input data .This data will be related to the created object by the given property.
    User need to pass the data in this key-value pair format: Property - array of data
    :var workflowData: Input data required for workflow creation. User need to pass the data in this key-value pair
    format:submitToWorkflow - true/false.
    NOTE: If the above option is "true" then workflow process template to be used for workflow creation should be
    specified in the preference (TypeName_default_workflow_template ) defined for the created object type.
    :var targetObject: Target to which created object will be pasted
    :var pasteProp: Property to be used to paste the created object to  the target
    """
    clientId: str = ''
    createData: CreateInput2 = None
    dataToBeRelated: PropertyValues = None
    workflowData: PropertyValues = None
    targetObject: BusinessObject = None
    pasteProp: str = ''


@dataclass
class CreateInput2(TcBaseObj):
    """
    CreateInput2 structure used to capture the inputs required for creation of a business object. This is a recursive
    structure containing the CreateInput(s) for any secondary(compounded) objects that might be created (e.g Item also
    creates ItemRevision and ItemMasterForm etc.)
    
    :var boName: Business Object type name
    :var propertyNameValues: Map of property name (key) and property values (values) in string format, to be set on new
    object being created. 
    Note: The calling client is responsible for converting the different property types (int, float, date .etc) to a
    string using the appropriate function(s) in the SOA client framework Property class.
    :var compoundCreateInput: CreateInput for compounded objects
    """
    boName: str = ''
    propertyNameValues: PropertyValues = None
    compoundCreateInput: CreateInputMap = None


@dataclass
class CreateOut2(TcBaseObj):
    """
    This is output structure for create operation including unique client identifier.
    
    :var clientId: Unique client identifier
    :var objects: List of tags representing objects that were created
    """
    clientId: str = ''
    objects: List[BusinessObject] = ()


@dataclass
class CreateResponse2(TcBaseObj):
    """
    This is response object structure for create operation.
    
    :var output: List of output objects representing objects that were created
    :var serviceData: Service data including partial errors that are mapped to the client id
    """
    output: List[CreateOut2] = ()
    serviceData: ServiceData = None


"""
CreateInputMap is a map of reference or relation property name to secondary CreateInput2 objects.
"""
CreateInputMap = Dict[str, List[CreateInput2]]


"""
PropertyValues is a map of property name (key) and property values (values) in string format.
"""
PropertyValues = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class GetPreferencesAtLocationsResponse(TcBaseObj):
    """
    Structure returned by the 'getPreferencesAtLocation' operation.
    
    :var data: Partial errors. Each partial error will contain the index of the related input data.
    :var responses: The preference description and category for the locations OOTB (out-of-the-box), Overlay or Site.
    In the other cases, those strings will be empty.
    """
    data: ServiceData = None
    responses: List[CompletePreference] = ()


@dataclass
class GetPreferencesResponse(TcBaseObj):
    """
    Contains the preference information (value and description if requested), as well as any potential partial errors.
    
    :var data: Contains the partial errors in the same order as in the input list.
    :var response: A list of complete preference information.
    Each structure holds all the preference definition ('PreferenceDefinition' structure) and its values
    ('PreferenceValue' structure).
    """
    data: ServiceData = None
    response: List[CompletePreference] = ()


@dataclass
class ImportPreferencesAtLocationDryRunIn(TcBaseObj):
    """
    A structure that contains all the parameters for the import dry run operation.
    
    :var location: Location where to simulate the import.
    :var fileTicket: Ticket of the input XML file in the transient volume uploaded by the client
    :var categoryNames: Vector of category names to be considered for import. No value means import all categories
    """
    location: PreferenceLocation = None
    fileTicket: str = ''
    categoryNames: List[str] = ()


@dataclass
class ImportPreferencesAtLocationDryRunResponse(TcBaseObj):
    """
    Response from importPreferencesAtLocationDryRun operation.
    
    :var serviceData: A ServiceData structure.
    The following partial errors will be returned:
     * An error 1759 will be returned if the preference name is empty. 
     * An error 1760 will be returned if the preference name is invalid ("*" for instance). 
     * An error 1700 will be returned if the preference does not exist.
     * An error 1751 will be returned if the specified object information does not correspond to any user, role or
    group.
     * An error 1752 will be returned if both an object and a location are specified for an entry.
     * An error 1753 will be returned if the specified location is invalid.
     * An error 1725 will be returned if the logged-in user does not have the requested permission to carry-out the
    operation.
     * An error 1728 will be returned in case the protection scope of the preference prevents a creation of one
    instance at the specified location.
    :var dbImpactedPreferences: List of preferences in the database that would be impacted by the import.
    :var conflictingPreferencesFromFile: Llist of preferences from the input file, for which conflict will arise. 
    Conflicts are cases in which either the preference already exists in the database and is different, or the
    preference does not exist in the database yet.
    """
    serviceData: ServiceData = None
    dbImpactedPreferences: List[CompletePreference] = ()
    conflictingPreferencesFromFile: List[CompletePreference] = ()


@dataclass
class ImportPreferencesAtLocationsIn(TcBaseObj):
    """
    A structure that contains all the parameters for the import operation.
    
    :var locations: A list of structures defining where the preferences are to be imported.
    :var fileTicket: Ticket of the input XML file in the transient volume uploaded by the client.
    :var categoryNames: List of category names to be considered for import. 
    No value means import all categories.
    :var importAction: Specifies what to do when the preference already exists.
    Valid values are:
    - "SKIP": to skip the import of this preference.
    - "OVERRIDE": to override the preference value at this location with the value specified in the input file.
    - "MERGE": to append the preference value from the input file to the preference value at the location. This is only
    possible if the preference is a multi-valued preference. Otherwise, this will result in the "SKIP" action for this
    preference. 
    
    
    If a wrong value is provided, the operation will revert to using the "SKIP" mode.
    """
    locations: List[PreferenceLocation] = ()
    fileTicket: str = ''
    categoryNames: List[str] = ()
    importAction: str = ''


@dataclass
class PreferenceDefinition(TcBaseObj):
    """
    Contains all the preference definition details.
    
    :var name: The name of the preference.
    :var category: The name of the category where the preference is sorted.
    A category is a logical group of preferences.
    :var description: The description associated with the preference.
    :var type: Type of the preference.
    Values are: 
    - 0 : String preference
    - 1: Logical preference
    - 2: Integer preference
    - 3: Double preference
    - 4: Date preference
    
    
    :var isArray: Determines if the preference is multi-valued.
    :var isDisabled: Determines if the preference is disabled. From Teamcenter 11.6.0, this is not supported. It is set
    to false always.
    :var protectionScope: Determines the protection scope of the preference, which is who is the lowest person (in the
    organization) who is allowed to provide values.
    Valid values are User, Role, Group, Site, System.
    :var isEnvEnabled: Determines if the value can be taken from an environment variable.
    :var isOOTBPreference: Determines if the preference is defined out-of-the-box. From Teamcenter 11.6.0, this is not
    supported since there is no differentiation between OOTB and non OOTB preferences. It is set to false always.
    """
    name: str = ''
    category: str = ''
    description: str = ''
    type: int = 0
    isArray: bool = False
    isDisabled: bool = False
    protectionScope: str = ''
    isEnvEnabled: bool = False
    isOOTBPreference: bool = False


@dataclass
class PreferenceLocation(TcBaseObj):
    """
    Defines where the preference is to be retrieved. 
    This structure is used by different operations with different possible values for the location parameter.
    
    :var object: Object where the preferences will be retrieved. This can be a User, Role or Group.
    This value can only be used if the location string is empty.  
    
    :var location: Name of the location.
    This value can only be specified if the object is not specified.
    """
    object: BusinessObject = None
    location: str = ''


@dataclass
class PreferenceResponseWithFileTicket(TcBaseObj):
    """
    Response that also contains a file ticket, which usage differs upon the operation returning it.
    
    :var serviceData: A ServiceData structure.  
    It will contain errors at indexes where errors have occurred. Error details will vary according to the calling
    operation, and it will be specified in its description.
    :var fileTicket: A ticket of a file in the transient volume uploaded by the client.
    The content of the file will depend on the calling operation, and it will be specified in its description.
    """
    serviceData: ServiceData = None
    fileTicket: str = ''


@dataclass
class PreferenceValue(TcBaseObj):
    """
    Contains the preference value and its location. 
    This structure is used by different operations with different possible values for the location parameter.
    
    :var values: The values associated with this preference.
    :var valueOrigination: Defines where the preference values are coming from.
    Valid values are:
    - "COTS": The value is the one defined by Siemens.
    - "Overlay":  The COTS value has been altered.
    - "Group": The value is defined at the group level.
    - "Role": The value is defined at the role level.
    - "User": The value is defined at the user level.
    - "Env": The value is coming from an environment variable.
    
    """
    values: List[str] = ()
    valueOrigination: str = ''


@dataclass
class PreferencesAtLocationIn(TcBaseObj):
    """
    Defines a list of preferences at a provided location.
    
    :var location: The desired location of the requested preferences.
    :var preferenceNames: A list of desired preference names.
    If the list is empty or its first element is "*", all the preferences for the specified locations (and only for
    preference instances at this location) are returned.
    """
    location: PreferenceLocation = None
    preferenceNames: List[str] = ()


@dataclass
class SetPreferences2In(TcBaseObj):
    """
    Structure of preference name and values.
    
    :var preferenceName: The name of the preference.
    :var values: The list of values.
    
    Even though the value can be of a type different than string, the conversion to string should be made using the
    'Property' class provided by the SOA framework.
    """
    preferenceName: str = ''
    values: List[str] = ()


@dataclass
class SetPreferencesAtLocationsIn(TcBaseObj):
    """
    Contains the preference definition, values and the location where to set the information.
    
    :var location: Structure defining where the preferences are to be set.
    
    The intent is that only the system administrator should create a preference. However, this could be needed from
    non-directly-related user interactions. Therefore, the decision to make a call to this operation is delegated to
    its caller.
    :var preferenceInputs: Structure that contains the preference name, definition and values.
    """
    location: PreferenceLocation = None
    preferenceInputs: SetPreferences2In = None


@dataclass
class SetPreferencesDefinitionIn(TcBaseObj):
    """
    Structure containing a 'PreferenceDefinition' structure and a list of strings representing the preference values at
    the site level.
    
    :var definition: A structure holding the preference definition (possibly simplified).
    :var values: The list of values at the site level.
    Even though the values can be of a type different than string, they should be converted to a string through the use
    of the 'Property' class from the SOA framework. 
    """
    definition: PreferenceDefinition = None
    values: List[str] = ()


@dataclass
class CompletePreference(TcBaseObj):
    """
    Composite structure completely representing a preference: its definition is captured in the PreferenceDefinition
    structure, and its values in the PreferenceValue structure.
    
    :var definition: A structure holding all the preference definition parameters.
    :var values: A structure holding the preference values.
    """
    definition: PreferenceDefinition = None
    values: PreferenceValue = None



from __future__ import annotations

from tcsoa.gen.StructureManagement._2015_10.Effectivity import EditRelStatusEffectivityInput, EditOccEffectivityInput, RemoveRelStatusEffectivityInput, RemoveOccEffectivitiesInput
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcService


class EffectivityService(TcService):

    @classmethod
    def removeOccurrenceEffectivities(cls, input: List[RemoveOccEffectivitiesInput]) -> ServiceData:
        """
        This operation removes effectivity objects from the specified BOMLine objects.
        
        Use cases:
        User can remove an occurrence effectivity by choosing Tools->Effectivity->Occurrence Effectivity, then in the
        Occurrence Effectivity dialog box, click Remove to clear all boxes. Click OK and Teamcenter removes the
        effectivity object from the selected occurrence. Any other occurrences sharing this effectivity retain their
        references to the effectivity object.
        """
        return cls.execute_soa_method(
            method_name='removeOccurrenceEffectivities',
            library='StructureManagement',
            service_date='2015_10',
            service_name='Effectivity',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def removeReleaseStatusEffectivity(cls, input: List[RemoveRelStatusEffectivityInput]) -> ServiceData:
        """
        This operation removes effectivity object from the specified released status
        
        Use cases:
        User can remove a release status effectivity by choosing Views->Effectivity->Revision Effectivity, then in the
        Effectivity dialog, select the corresponding release status and click Delete.
        """
        return cls.execute_soa_method(
            method_name='removeReleaseStatusEffectivity',
            library='StructureManagement',
            service_date='2015_10',
            service_name='Effectivity',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def editOccurrenceEffectivity(cls, input: List[EditOccEffectivityInput]) -> ServiceData:
        """
        This operation updates effectivity object for the specified BOMLine
        
        Use cases:
        User can configure a release status by choosing Tools->Effectivity->Revision Effectivity, then in the
        Effectivity dialog, select the corresponding release status and click create. Define Unit/Date range to set
        effectivity on the release status.
        """
        return cls.execute_soa_method(
            method_name='editOccurrenceEffectivity',
            library='StructureManagement',
            service_date='2015_10',
            service_name='Effectivity',
            params={'input': input},
            response_cls=ServiceData,
        )

    @classmethod
    def editReleaseStatusEffectivity(cls, input: List[EditRelStatusEffectivityInput]) -> ServiceData:
        """
        This operation updates effectivity object for the specified released status
        
        Use cases:
        - User can edit a release status by choosing Views->Effectivity->Revision Effectivity, then in the Revision
        Effectivity dialog box, select the corresponding release status effectivity and click Edit. Modify Unit/Date
        range to update effectivity on the release status.
        - User can also edit release status effectivity in My Teamcenter. Double-click the item status and change the
        displayed value.
        
        """
        return cls.execute_soa_method(
            method_name='editReleaseStatusEffectivity',
            library='StructureManagement',
            service_date='2015_10',
            service_name='Effectivity',
            params={'input': input},
            response_cls=ServiceData,
        )



from __future__ import annotations

from tcsoa.gen.Reports._2007_01.CrfReports import GetReportDefinitionsResponse
from typing import List
from tcsoa.gen.Reports._2008_06.CrfReports import ReportsCriteria2
from tcsoa.base import TcService


class CrfReportsService(TcService):

    @classmethod
    def getReportDefinitions(cls, inputCriteria: List[ReportsCriteria2]) -> GetReportDefinitionsResponse:
        """
        Retrieves a set of report definitions that meet the specified criteria.
        
        Use cases:
        Document set of user level use cases, should describe how user interacts with this operation to accomplish the
        goal.
        """
        return cls.execute_soa_method(
            method_name='getReportDefinitions',
            library='Reports',
            service_date='2008_06',
            service_name='CrfReports',
            params={'inputCriteria': inputCriteria},
            response_cls=GetReportDefinitionsResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BusinessObject
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class MapSrchCriteriaToLinesInputInfo(TcBaseObj):
    """
    Input object for mapSrchCriteriaToLines action.
    
    :var criteria: Search criteria object which needs to be interpreted. The search criteria objects can be of type
    ApprSearchCriteria or its sub types.
    :var contexts: Represents the context under which the objects will be found. The valid type of this parameter is
    BOM line or its sub types.
    """
    criteria: BusinessObject = None
    contexts: List[BusinessObject] = ()


@dataclass
class MapSrchCriteriaToLinesResponse(TcBaseObj):
    """
    Response object for mapSrchCriteriaToLines action.
    
    :var output: Vector of type SrchCriteriaToLinesMap which contains the search criteria and BOMLines.
    :var serviceData: Contains general exceptions  if occurred during mapSrchCriteriaToLines action.
    """
    output: List[SrchCriteriaToLinesMap] = ()
    serviceData: ServiceData = None


@dataclass
class SaveOGLinesInSrchCriteriaInputInfo(TcBaseObj):
    """
    Input object for saveOGLinesInSrchCriteria action.
    
    :var selectedNodes: The BOM line occurrences from the Occurrence Group structure which are in the selected state..
    Valid values are Occurrence Group BOP lines or normal BOM lines.
    :var unselectedNodes: The BOM line occurrences from the Occurrence Group structure which are in the unselected
    state.. Valid values are Occurrence Group BOP lines or normal BOM lines.
    :var scopeAttribute: Tells whether to create scope search criteria object (Fnd0ApprSchCriteriaScpAttr) or select
    state search criteria object( ApprSearchCriteriaSlctState). Value True means an object of scope search criteria
    will be returned.
    """
    selectedNodes: List[BusinessObject] = ()
    unselectedNodes: List[BusinessObject] = ()
    scopeAttribute: bool = False


@dataclass
class SaveOGLinesInSrchCriteriaResponse(TcBaseObj):
    """
    Output response object for saveOGLinesInSrchCriteria action.
    
    :var srchCriterias: Vector of objects of ApprSearchCriteria (its appropreate sub types) created by this operation.
    :var serviceData: Contains general exceptions if occurred during creation of Search Criteria object.
    """
    srchCriterias: List[BusinessObject] = ()
    serviceData: ServiceData = None


@dataclass
class SrchCriteriaToLinesMap(TcBaseObj):
    """
    Structure created to maintain a relation between search criteria and BOMLines.
    
    :var srchCriteria: The interpreted search criteria.
    :var selectedNodes: Selected BOMlines which were stored in the search criteria.
    :var unselectedNodes: Unselected BOMlines which were stored in the search criteria.
    """
    srchCriteria: BusinessObject = None
    selectedNodes: List[BusinessObject] = ()
    unselectedNodes: List[BusinessObject] = ()



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.Workflowaw._2020_12.Workflow import UpdateTaskAssignmentsReponse, TaskAssignmentsInput, TaskAssignmentsResponse, UpdateTaskAssignmentsInput
from tcsoa.base import TcService


class WorkflowService(TcService):

    @classmethod
    def getWorkflowTaskAssignments(cls, inData: List[TaskAssignmentsInput]) -> TaskAssignmentsResponse:
        """
        Get the assigments for all tasks or future tasks of the process (EPMJob) for which input EPMTask or
        EPMTaskTemplate belongs to.The assignments may be for EPMTask responsible party (User or ResourcePool) or for
        Signoffs (GroupMember or ResourcePool). These are presented as Reviewers, Acknowlegers, Notifiers (for Signoff)
        or Assignees (for task responsible party) on individual tasks.It also contains information about the origin of
        the task assigments. For example, EPMSignoffProfile, ResourcePool or Participant object.
        
        Use cases:
        Details provided by this operation are be displayed in the assignments panel to get an overview of the task
        assignments of all tasks for a workflow.
        """
        return cls.execute_soa_method(
            method_name='getWorkflowTaskAssignments',
            library='Internal-Workflowaw',
            service_date='2020_12',
            service_name='Workflow',
            params={'inData': inData},
            response_cls=TaskAssignmentsResponse,
        )

    @classmethod
    def updateWorkflowTaskAssignments(cls, inData: List[UpdateTaskAssignmentsInput]) -> UpdateTaskAssignmentsReponse:
        """
        Updates the assigments for all the tasks sent as input parameter. The assignments may be for EPMTask
        responsible party (User or ResourcePool) or for Signoffs (GroupMember or User or ResourcePool. These are
        presented as Reviewers, Acknowlegers, Notifiers (for Signoff) or Assignees (for EPMTask responsible party) on
        individual tasks. Apart from task assignments, it also contains the assignees of dynamic Participant and\or
        members of the signoff profile (EPMSignoffProfile) that are changed and that needs to be updated.
        
        Use cases:
        This operation provides details to update the task assignments , members of the signoff profiles
        (EPMSignoffProfile) and the dynamic participant (Participant) assignees.
        """
        return cls.execute_soa_method(
            method_name='updateWorkflowTaskAssignments',
            library='Internal-Workflowaw',
            service_date='2020_12',
            service_name='Workflow',
            params={'inData': inData},
            response_cls=UpdateTaskAssignmentsReponse,
        )



from __future__ import annotations

from tcsoa.gen.Classification._2016_09.Classification import FindValuesResponse, FindValueInput
from typing import List
from tcsoa.base import TcService


class ClassificationService(TcService):

    @classmethod
    def findValues(cls, findValueInputs: List[FindValueInput]) -> FindValuesResponse:
        """
        This operation returns all values available for an attribute in the context of where it is called.
        For example, it returns all the values of a particular attribute in a given class or in the entire database. 
        Some examples are:
        - Find all the available length values in the Sheet Metal Screw class. It will return responses such as 5mm,
        6mm, 7.5 mm, etc.
        - Find all the values for the Supplier attribute. It will return all values in all classes providing a list of
        all used suppliers stored in the classification.
        
        
        
        The operation can take other attribute values into consideration to narrow the results, for example:
        - Find all the available length values in the Sheet Metal Screw class where the diameter is 4mm. It will return
        responses such as 5mm, 6mm, etc.
        
        
        
        Use cases:
        Teamcenter Classification displays the List of Values dialog box containing a list of attribute values stored
        for the input attribute, their count and unit system in which those values are stored. This operation helps
        user to search for such stored values for multiple input attributes.
        The search can be constrained by setting other attribute values, the operations returns only the attribute
        values that are valid given the current search criteria (helping the user to efficiently narrow down the search
        and choose valid values that will find classified objects).
        """
        return cls.execute_soa_method(
            method_name='findValues',
            library='Classification',
            service_date='2016_09',
            service_name='Classification',
            params={'findValueInputs': findValueInputs},
            response_cls=FindValuesResponse,
        )



from __future__ import annotations

from tcsoa.gen.BusinessObjects import BOMLine
from typing import List
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class AssignChildLineOccTypes(TcBaseObj):
    """
    This structure defines child lines to be assigned with its occurrence type and client id.
    
    :var lineToAssign: lineToAssign
    :var occType: occurrence type for new assigned line. it can be empty.
    """
    lineToAssign: BOMLine = None
    occType: str = ''


@dataclass
class AssignChildLinesOutput(TcBaseObj):
    """
    Holds the response from assignChildLines
    
    :var newLines: new BOMLines created as a result of assign operation under newParentLine
    :var linesWithoutPreds: a subset new bomlines that do have incoming flows.
    """
    newLines: List[BOMLine] = ()
    linesWithoutPreds: List[BOMLine] = ()


@dataclass
class AssignChildLinesParameter(TcBaseObj):
    """
    This structure provides a set of input values for assignLines operation.
    
    :var newParentLine: new parent line under which the bomlines need to be assigned
    :var linesToAssign: Array of AssignChildLineOccType
    :var copyPreds: should predecessor relationship be copied over from old parent?
    :var copyOccTypeFromSource: occurrence type to be used for newly assigned lines.
    """
    newParentLine: BOMLine = None
    linesToAssign: List[AssignChildLineOccTypes] = ()
    copyPreds: bool = False
    copyOccTypeFromSource: bool = False


@dataclass
class AssignChildLinesResponse(TcBaseObj):
    """
    Holds the response from assignChildLines
    
    :var assignOutput: array of AssignChildLinesOutput
    :var serviceData: Exceptions from internal processing returned as PartialErrors
    """
    assignOutput: List[AssignChildLinesOutput] = ()
    serviceData: ServiceData = None



from __future__ import annotations

from tcsoa.gen.Translation._2007_06.TranslationManagement import CreateTranslationRequestArgs, CreateTranslationRequestResponse
from typing import List
from tcsoa.base import TcService


class TranslationManagementService(TcService):

    @classmethod
    def createTranslationRequest(cls, inputs: List[CreateTranslationRequestArgs]) -> CreateTranslationRequestResponse:
        """
        Create a translation request within Teamcenter for use with translation services.
        This operation creates a ETSTranslationRequest object in the Teamcenter database.    
        
        
        Use cases:
        The ETS application provides the ability to process requests in an asynchronous fashion thus removing the
        processing burden from the clients to a provisioned machine dedicated to processing these requests.
        """
        return cls.execute_soa_method(
            method_name='createTranslationRequest',
            library='Translation',
            service_date='2007_06',
            service_name='TranslationManagement',
            params={'inputs': inputs},
            response_cls=CreateTranslationRequestResponse,
        )



from __future__ import annotations

from typing import Dict, List
from tcsoa.gen.Internal.AWS2._2016_03.Finder import SearchFilterField, SearchSortCriteria, ObjectsGroupedByProperty
from tcsoa.gen.Internal.AWS2._2018_05.Finder import SearchCursor
from tcsoa.gen.Server import ServiceData
from tcsoa.base import TcBaseObj
from dataclasses import dataclass
from tcsoa.gen.Internal.AWS2._2017_12.Finder import SearchFilter4


@dataclass
class SaveColumnConfigData2(TcBaseObj):
    """
    This structure contains column configuration details and a list of column definitions information.
    
    :var scope: The scope value is used to apply the UI configurations for that scope. Supported values are :
    "LoginUser" - Returns the current configuration defined for the current login user. 
    Note: Other scope values "Site", "Group" and "Role" are not supported.
    :var scopeName: The name of a valid Teamcenter scope.
    For example "Group" scope, scopeName can be "dba","Engineering". 
    For "LoginUser" scope, this value should be empty.
    :var clientScopeURI: The unique name of the client scope containing configurations.
    Supported values are: 
    "Awp0SearchResults" for global search
    "fnd0Inbox" for inbox
    "fnd0alltasks" for all inbox tasks
    "fnd0mytasks" for my inbox tasks
    "fnd0surrogatescope" for surrogate inbox tasks
    "Fnd0Report" for report
    "Awp0ObjectNavigation" for object navigation
    "Awp0AdvancedSearch" for advanced search
    :var columnConfigId: The unique identifier for the column configuration.
    Ex: "searchResultsColConfig" for search location.
    :var columns: Ordered list of column information.
    """
    scope: str = ''
    scopeName: str = ''
    clientScopeURI: str = ''
    columnConfigId: str = ''
    columns: List[ColumnDefInfo3] = ()


@dataclass
class ColumnConfig3(TcBaseObj):
    """
    This structure contains information for a column configuration within a client scope URI. It contains a unique
    column config id, a list of column definition information, and the operation type used to finalize the columns.
    
    :var columnConfigId: The unique identifier of the column configuration.
    :var operationType: The operation that was used to finalize the columns to be returned back. Supported values are:
    "Intersection", "Union" and "Configured".
    :var columns: List of column details.
    """
    columnConfigId: str = ''
    operationType: str = ''
    columns: List[ColumnDefInfo3] = ()


@dataclass
class SearchInput3(TcBaseObj):
    """
    A structure containing input search criteria.
    
    :var providerName: The name of the search provider. This is the RuntimeBusinessObject type name.
    :var searchCriteria: A map (string, string) used to perform the search. 
    For full text search, the key is "searchString", the value is "*".
    For advanced search, the key can be
    "queryID" ,
    "quickSearchAttributeValue",
    "searchID",
    "typeOfSearch", "QUICK_SEARCH" or "ADVANCED_SEARCH", the value is unique identifier.
    :var searchFilterFieldSortType: The sorting type to use to order the search filter categories in the response.
    Supported values are: "Alphabetical" and "Priority".
    :var attributesToInflate: A list of attributes to inflate (extract the details).
    :var internalPropertyName: The internal name of the property used for grouping.
    :var columnFilters: A list of filters to be applied on the columns.
    :var cursor: Cursor object used in the search.
    :var focusObjUid: Uid of the Object to focus to
    :var pagingType: The type of paging operation. Valid values are GetCurrentPage, GetNextPage, GetPreviousPage.
    :var startIndex: The start index to return the search results.
    :var maxToReturn: The maximum number of search results to return.If zero, no search results are returned; however,
    other information such as total results found and filters details are returned.
    :var maxToLoad: The maximum number of search results to load.
    :var searchFilterMap6: A map (string, list of SearchFilter4) containing the list of search filters for each search
    filter field.The key in the map is the property name that represents the filter category. It is in the format
    "TypeName.PropertyName". For example: WorkspaceObject.release_statuses, the value in the map is the value of the
    search filter.
    :var searchSortCriteria: The criteria to use to sort the results.
    """
    providerName: str = ''
    searchCriteria: StringMap3 = None
    searchFilterFieldSortType: str = ''
    attributesToInflate: List[str] = ()
    internalPropertyName: str = ''
    columnFilters: List[FilterCriteriaInput] = ()
    cursor: SearchCursor = None
    focusObjUid: str = ''
    pagingType: str = ''
    startIndex: int = 0
    maxToReturn: int = 0
    maxToLoad: int = 0
    searchFilterMap6: SearchFilterMap6 = None
    searchSortCriteria: List[SearchSortCriteria] = ()


@dataclass
class SearchResponse6(TcBaseObj):
    """
    A service response structure containing search results and column configurations.
    
    :var searchResultsJSON: Search results in JSON Format. With this, Client can directly bind the view model
    properties JSON to the view.
    :var totalFound: Total number of view model business objects found.
    :var additionalSearchInfoMap: A generic map (string/list of strings) that can be used to send additional search
    information to the consumer. 
    
    Supported key values are:
    searchTermsToHighlight - used to indicate which Search Terms need to be highlighted.
    :var cursor: Cursor for the results returned so far.
    :var searchFilterCategoriesUnpopulated: A list of empty search filter categories.
    :var totalLoaded: Total number of view model business objects loaded.
    :var searchFilterMap6: A map (string, list of SearchFilter4) containing the list of search filters for each search
    filter field based on the search results.
    :var searchFilterCategories: A list of search filter categories ordered by filter priority.
    :var defaultFilterFieldDisplayCount: The default number of search filter categories to display.
    :var objectsGroupedByProperty: The structure containing an internal property name and a map of objects to the
    property group id. It also contains a list of unmatched objects which do not match any group.
    :var columnConfig: Effective column configuration for the client scope URI.
    :var serviceData: The service data object.
    :var endIndex: Cursor end position for the results returned so far. This value, 'endIndex', correlates with
    'startindex' in SearchInput.
    """
    searchResultsJSON: str = ''
    totalFound: int = 0
    additionalSearchInfoMap: StringVectorMap2 = None
    cursor: SearchCursor = None
    searchFilterCategoriesUnpopulated: List[SearchFilterField] = ()
    totalLoaded: int = 0
    searchFilterMap6: SearchFilterMap6 = None
    searchFilterCategories: List[SearchFilterField] = ()
    defaultFilterFieldDisplayCount: int = 0
    objectsGroupedByProperty: ObjectsGroupedByProperty = None
    columnConfig: ColumnConfig3 = None
    serviceData: ServiceData = None
    endIndex: int = 0


@dataclass
class ColumnDefInfo3(TcBaseObj):
    """
    Contains details about a specific column. This includes the type of object for which the column is applicable, the
    name of the property displayed in the column, a flag indicating if the column should be used to order information
    displayed in the client, pixel width of the column, a flag indicating if the column should be hidden and the column
    sort order, fliters applied on a column, filterDefinition and the datatype of the property.
    
    :var displayName: The display name for the value displayed in the column header.
    :var typeName: The business object type for the value displayed in the column. This can be any valid Teamcenter
    business object type.
    :var isFilteringEnabled: If true, column filtering is enabled on this column; otherwise, column filtering is not
    supported on this column.
    :var dataType: The data type of the property for the given column.
    :var isFrozen: If true, the table has columns frozen until this column; otherwise, this column is not frozen.
    :var propertyName: The property name for the value displayed in the column.
    :var pixelWidth: The pixel width for the column. Valid pixel widths are integer values between 1 and 500.
    :var columnOrder: The column order value is used to arrange the columns in order.
    :var hiddenFlag: If true, the column should be hidden on the client user interface.
    :var sortPriority: Sort priority set on column helps identify the order in which the columns should be used during
    sort. Sort priority value will be zero for columns not marked for sorting.
    :var sortDirection: How the columns are sorted. Supported values are: "Ascending" and "Descending". This value will
    be empty if the column is not marked for sorting.
    :var filterValue: The filter criteria that is applied for current operation.
    :var filterDefinitionKey: This string that represents a custom filter definition which the clients can define in
    the client view model.
    """
    displayName: str = ''
    typeName: str = ''
    isFilteringEnabled: bool = False
    dataType: str = ''
    isFrozen: bool = False
    propertyName: str = ''
    pixelWidth: int = 0
    columnOrder: int = 0
    hiddenFlag: bool = False
    sortPriority: int = 0
    sortDirection: str = ''
    filterValue: FilterCriteriaResponse = None
    filterDefinitionKey: str = ''


@dataclass
class FilterCriteriaInput(TcBaseObj):
    """
    A structure representing the input filter criteria for columns.
    
    :var columnName: The internal name of the property on which column filtering is applied.
    :var operation: This operation specifies the type of column filter operation applied. The supported operations are:
    "CONTAINS", "EQUAL", "GREATER THAN", "LESS THAN" and "BETWEEN".
    :var values: The filter values are the criteria entered by the user on the client. These are not a fixed set of
    values like operation.
    """
    columnName: str = ''
    operation: str = ''
    values: List[str] = ()


@dataclass
class FilterCriteriaResponse(TcBaseObj):
    """
    A response structure representing the column filter criteria.
    
    :var operation: This operation specifies the type of column filter operation applied. The supported operations are
    "CONTAINS", "EQUAL", "GREATER THAN", "LESS THAN" and "BETWEEN".
    :var values: These are the filter values saved in the database from the previous filter use case executed by the
    user.
    """
    operation: str = ''
    values: List[str] = ()


"""
A map containing a list of search filters for each search filter field.
"""
SearchFilterMap6 = Dict[str, List[SearchFilter4]]


"""
Map from string to string.
"""
StringMap3 = Dict[str, str]


"""
A map of string to list of strings.
"""
StringVectorMap2 = Dict[str, List[str]]



from __future__ import annotations

from tcsoa.base import TcBaseObj
from dataclasses import dataclass


@dataclass
class StateNameValue(TcBaseObj):
    """
    This structure is used to hold a single name/value pair.
    
    :var name: The name of the UserSession state property.
    :var value: The value of the state property.
    """
    name: str = ''
    value: str = ''



from __future__ import annotations

from tcsoa.gen.Internal.Cad._2008_03.DataManagement import ExportConfiguredNXAssemblyResponse, ExportConfiguredNXAssemblyInfo
from typing import List
from tcsoa.base import TcService


class DataManagementService(TcService):

    @classmethod
    def exportConfiguredNXAssembly(cls, inputs: List[ExportConfiguredNXAssemblyInfo]) -> ExportConfiguredNXAssemblyResponse:
        """
        This operation is used by the command line utility ps_exportconfignxassembly and from the Teamcenter Rich
        Client Structure Manager, menu item Tools -> Export Configured UGNX Assembly.  This feature is enabled in
        Teamcenter Structure Manager only if the preference TC_ ExportConfigUGNXAssembly is true.  The assembly is
        configured using the given revision rule and saved variant rule.  The goal is to export a configured assembly
        structure to native NX.   This feature is supported by wrapping the existing functionality in a published BOM
        ITK. The client UI is designed to enable to user to initiate the export operation from the Structure Manager.
        
        Use cases:
        Usecase 1: 
        Initiating export configured UGNX assembly from the Teamcenter Rich Client.
        User configures a UGNX assembly in Structure Manager using revision rule and variant rule. 
        User selects the top BOMLine and initiates Tools->ExportConfiguredUGNXAssembly
        Upon completion of the operation, the entire configured structure in Structure Manager is exported to the temp
        directory.
        The precondition to this usecase is that the preference TC_ ExportConfigUGNXAssembly has to be true.
        
        Use case 2: 
        Initiating export configured UGNX assembly from the with the command line utility ps_ exportconfignxassembly
        User runs the command line utility ps_exportconfigurednxassembly with the following arguments: ItemId of the
        assembly, ItemRevId of the assembly, revision rule name, and variant rule name. The optional arguments are
        -display, -debug=<y or n> and the list of excluded items.
        Upon completion of the operation, the assembly and all its configured children are exported to the temp
        directory.
        """
        return cls.execute_soa_method(
            method_name='exportConfiguredNXAssembly',
            library='Internal-Cad',
            service_date='2008_03',
            service_name='DataManagement',
            params={'inputs': inputs},
            response_cls=ExportConfiguredNXAssemblyResponse,
        )



import os
from typing import List

from tcsoa.fcc.fcc_client_proxy import FccClientProxy, FMSException


class FccWrapper:
    def __init__(self):
        self.client_proxy = FccClientProxy()

    def register_tickets(self, tickets: List[str]) -> List[str]:
        if tickets is None or len(tickets) == 0:
            raise FMSException('Error -3002: ArgumentError - pass a non-empty list of tickets')
        result = self.client_proxy.fcc_register_tickets(tickets)
        return result.ticket_uids

    def unregister_tickets(self, uids: List[str]):
        if uids is None or len(uids) == 0:
            raise FMSException('Error -3002: ArgumentError - pass a non-empty list of UIDs')
        self.client_proxy.fcc_unregister_tickets(uids)

    def download_files_from_plm(self, policy: str, uids: List[str]) -> List[str]:
        if uids is None or len(uids) == 0:
            raise FMSException('Error -3002: ArgumentError - pass a non-empty list of UIDs')
        result = self.client_proxy.fcc_download_files_from_plm(policy, uids, None, None)
        return result.local_files

    def download_file_to_location(self, policy: str, uid: str, target_dir: str, file_name: str):
        self.client_proxy.fcc_download_file_to_location(policy, uid, None, None, target_dir, file_name)

    def download_files_to_location(self, policy: str, uids: List[str], target_dir: str, file_names: List[str]):
        """
        Downloads multiple files from Teamcenter to the specified directory.
        IMPORTANT: the uids-list and file_names-list must have the same length

        :param policy: unknown - in SOA-SDK, the value "IMD" is used everywhere
        :param uids: the tickets, which should have been registered with FileManagementService#getFileReadTickets
        :param target_dir: the directory the files should be loaded into
        :param file_names: the file names of each of the files.
        """
        if uids is None or len(uids) == 0:
            raise FMSException('Error -3002: ArgumentError - pass a non-empty list of UIDs')
        if not target_dir or not os.path.isdir(target_dir):
            raise FMSException('Error -3002: ArgumentError - please provide a valid, existing path for the files to be downloaded')
        self.client_proxy.fcc_download_files_to_location(policy, uids, None, None, target_dir, file_names)

    def upload_files_to_plm(self, uids: List[str], file_paths: List[str]) -> List[str]:
        if uids is None or len(uids) == 0:
            raise FMSException('Error -3002: ArgumentError - pass a non-empty list of UIDs')
        result = self.client_proxy.fcc_upload_files_to_plm(uids, None, None, file_paths)
        return result.volume_ids

    def rollback_files_uploaded_to_plm(self, uids: List[str], volume_ids: List[str]):
        self.client_proxy.fcc_rollback_files_uploaded_to_plm(uids, volume_ids)



from tcsoa.gen.Configuration._2010_04.services import SmartUiBldrService as imp0
from tcsoa.base import TcService


class SmartUiBldrService(TcService):
    getPropertiesForSelections = imp0.getPropertiesForSelections
    getResultForSelections = imp0.getResultForSelections
    getValuesForComponent = imp0.getValuesForComponent
    listForerunnersForComponents = imp0.listForerunnersForComponents
    listObjectsForComponents = imp0.listObjectsForComponents
    listValuesForComponents = imp0.listValuesForComponents
    ungetValuesForComponents = imp0.ungetValuesForComponents



from __future__ import annotations

from typing import List
from tcsoa.gen.Internal.ProjectManagement._2011_06.ScheduleManagement import ScheduleModifyContainer
from tcsoa.base import TcBaseObj
from tcsoa.gen.Internal.ProjectManagement._2009_10.ScheduleManagement import TranslatorException
from dataclasses import dataclass


@dataclass
class TranslatorResponseScheduleModifyContainer(TcBaseObj):
    """
    Translator schedule modify response structure
    
    :var translatorResponseScheduleModifyContainerSub: list of sub translator schedule modify response containers
    """
    translatorResponseScheduleModifyContainerSub: List[TranslatorResponseScheduleModifyContainerSub] = ()


@dataclass
class TranslatorResponseScheduleModifyContainerSub(TcBaseObj):
    """
    structure used internally on the server side to share objects between C++ and Java
    
    :var scheduleUid: the schedule uid
    :var scheduleModifyContainer: list of schedule modify structures
    :var translatorExceptions: list of translator exceptions
    """
    scheduleUid: str = ''
    scheduleModifyContainer: List[ScheduleModifyContainer] = ()
    translatorExceptions: List[TranslatorException] = ()



from __future__ import annotations

from tcsoa.gen.Internal.ActiveWorkspaceBom._2020_12.OccurrenceManagement import OccTypesInputData, RemoveLevelInputData, RemoveLevelResponse, InfoForInsertLevelData, ResetWorkingContextResp, InfoForInsertLevelResp, ResetWorkingContextInputData, InsertLevelResponse2, OccTypesResp, InsertLevelInputData2
from tcsoa.base import TcService


class OccurrenceManagementService(TcService):

    @classmethod
    def getAllowedOccurrenceTypes(cls, inputData: OccTypesInputData) -> OccTypesResp:
        """
        This operation retrieves a list of allowed occurrence type names. The operation also supports filtering of
        allowed occurrence types based on Teamcenter preference MEDisplayOccurrenceType.
        
        Use cases:
        Use case 1: You want to set an occurrence type on Awb0Element or BOMLine from Active Workspace. You can
        retrieve a list of valid occurrence types using this operation before setting a specific occurrence type.
        
        Use case 2: You want to copy Awb0Element or BOMLine and paste on other Awb0Element or BOMLine using paste
        sub-menu having allowed occurrence types. This operation can be used to create list of the sub-menu options.
        
        Use case 3: You want to assign a Resource (Item/Item Revision) from classification to an Operation (Item/Item
        Revision). This operation can be used to get list of allowed occurrence types.
        """
        return cls.execute_soa_method(
            method_name='getAllowedOccurrenceTypes',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_12',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=OccTypesResp,
        )

    @classmethod
    def getInfoForInsertLevel(cls, getInfoForInsertLevelIn: InfoForInsertLevelData) -> InfoForInsertLevelResp:
        """
        This operation retrieves valid parent type name(s) for given selectedElements based on preferences
        "TCAllowedParentTypes_<ChildItemType>" and "TCAllowedChildTypes_<ParentItemType>". Response contains preferred
        type as first value in preference "TCAllowedChildTypes_<ParentItemType>". When preference does not exist then
        parent of selectedElements is preferred parent type. 
        The operation returns allowed parent occurrence revision type name(s) for selectedElements when input has
        fetchAllowedOccRevTypes set to true.
        If fetchAllowedOccRevTypes is set and TCAllowedParentOccRevTypes_<ChildItemType> do not exists, then the
        operation returns preferred type as parent occurrence type of selectedElements; otherwise preferred type is the
        first type specified in TCAllowedParentOccRevTypes_<ChildItemType> preference.
        """
        return cls.execute_soa_method(
            method_name='getInfoForInsertLevel',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_12',
            service_name='OccurrenceManagement',
            params={'getInfoForInsertLevelIn': getInfoForInsertLevelIn},
            response_cls=InfoForInsertLevelResp,
        )

    @classmethod
    def insertLevel2(cls, insertLevelInput: InsertLevelInputData2) -> InsertLevelResponse2:
        """
        This operation removes elements from existing parent and adds them under a new parent for given
        objectToBeInserted . The content is saved as part of this operation.
        """
        return cls.execute_soa_method(
            method_name='insertLevel2',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_12',
            service_name='OccurrenceManagement',
            params={'insertLevelInput': insertLevelInput},
            response_cls=InsertLevelResponse2,
        )

    @classmethod
    def removeLevel(cls, inputData: RemoveLevelInputData) -> RemoveLevelResponse:
        """
        This operation removes the input Awb0Element objects from content and adds its children to the parent of input
        Awb0Element object.
        """
        return cls.execute_soa_method(
            method_name='removeLevel',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_12',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=RemoveLevelResponse,
        )

    @classmethod
    def resetUserWorkingContextState(cls, inputData: ResetWorkingContextInputData) -> ResetWorkingContextResp:
        """
        This operation reset the user's working state. User working state includes configuration and application data
        such as diagram layout and visualization aspects. If user working state does not exist then a new user working
        state with default configuration specified in Awb0ProductContextInfo is created.
        """
        return cls.execute_soa_method(
            method_name='resetUserWorkingContextState',
            library='Internal-ActiveWorkspaceBom',
            service_date='2020_12',
            service_name='OccurrenceManagement',
            params={'inputData': inputData},
            response_cls=ResetWorkingContextResp,
        )



from __future__ import annotations

from tcsoa.gen.Internal.DebugMonitor._2019_06.DebugLogging import Logflags, DebugLoggingResponse
from tcsoa.base import TcService


class DebugLoggingService(TcService):

    @classmethod
    def startLogging(cls, logFlags: Logflags) -> DebugLoggingResponse:
        """
        This operation enables logging of the debug log flags that are passed through logFlags.
        """
        return cls.execute_soa_method(
            method_name='startLogging',
            library='Internal-DebugMonitor',
            service_date='2019_06',
            service_name='DebugLogging',
            params={'logFlags': logFlags},
            response_cls=DebugLoggingResponse,
        )

    @classmethod
    def stopLogging(cls) -> DebugLoggingResponse:
        """
        This operation disables logging of the selected log type, collects all the generated log files and creates a
        zip file of the logs. It either returns a FMS transient file ticket of a zip file containg all the relavent log
        files, or a message informing the user to contact the system adminstrator for access to the zip file.
        """
        return cls.execute_soa_method(
            method_name='stopLogging',
            library='Internal-DebugMonitor',
            service_date='2019_06',
            service_name='DebugLogging',
            params={},
            response_cls=DebugLoggingResponse,
        )
